/* Copyright (c) 2015, Electric Cloud, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* (As an historical note, the above license instantiates
 * the "BSD 2-Clause License" template that was presented
 * by the Open Source Initiative at
 *
 *   http://www.opensource.org/licenses/bsd-license.php
 *
 * on 2012-02-23.)
 */

/* Summary:
 *
 * When combined with a calling convention, the following declarations
 * define a binary interface to a virtual POSIX-like system, one which
 * might or might not use an actual POSIX system underneath.
 *
 * Rationale:
 *
 * There are modest costs to such virtualization, but also benefits:
 *
 *   1. Portability and compatibility: the same interface may be supported
 *   by multiple versions of an underlying implementation, on multiple real
 *   systems.  Furthermore, an implementation may support multiple versions
 *   of this interface simultaneously, meaning that it could dynamically
 *   load two libraries that were independently written for incompatible
 *   versions of this interface, and support them both at the same time.
 *
 *   2. Instrumentation: you can easily monitor interface calls in order
 *   to trace, debug, or audit operations that make use of this interface.
 *
 *   3. Sandboxing: you can get an effect similar to "chroot", though without
 *   the security aspects--we do not attempt to prevent a client from bypassing
 *   this interface and directly contacting any underlying software layer(s).
 *   Such sandboxing might be of use in compute service clouds and in browsers,
 *   though again please note that you must trust the client software.
 *
 * Semantics:
 *
 * Where practical, and where not otherwise documented, the semantics should
 * be those of POSIX.  If not, then please feel free to report a bug.
 *
 * If there are two variants of file I/O, one of which supports only 32-bit
 * offset/size figures and one of which supports 64-bit ones, then the 64-bit
 * one should be used.  This interface is in terms of 64-bit offset and size,
 * so if only a 32-bit implementation is available, then promote the values.
 *
 * What is an "Interface"?:
 *
 * By an "interface", we mean a particular way of invoking functionality
 * and accessing state that prevents the client from becoming dependent
 * upon the details of a particular implementation.
 *
 * Our interface conventions are described in more detail below, but
 * in short: an "interface" consists of a pair of structs, one defining
 * a state handle and one defining the pointers functions that can be
 * used on that handle.  Calls are always indirect.  We avoid using
 * linker symbols, except for one top-level symbol as described in
 * "vrpsx_identified_function.h".
 *
 * For example, there is a "vrpsx_directory" interface expressing the
 * current state of an iteration over the entries in a directory,
 * with state handle "vrpsx_directory_t" and function pointer table
 * "vrpsx_directory_vtbl_t".  The latter table contains a "readdir"
 * function pointer that can be used to get the next directory entry.
 *
 * But you would obtain that "vrpsx_directory" interface from the
 * (large) "vrpsx_system" interface that corresponds to the collection
 * of POSIX system calls that we currently virtualize.  For example,
 * "vrpsx_system_vtbl_t" contains an "opendir" function pointer that
 * corresponds to the "opendir" POSIX system call.
 *
 * You could modify the interface to include more system calls by adding
 * more such function pointers to that structure.  Doing so would produce
 * a new interface, and you would need to define new interface identifiers
 * as described in "vrpsx_identified.h" and "vrpsx_identified_function.h".
 *
 * Interface Type Names:
 *
 * By convention we append "_struct" to the base interface name to obtain
 * the struct name for the state handle, and "_t" to obtain its typedef.
 *
 * Likewise, we append "_vtbl_struct" to the base interface name
 * to obtain the struct name for the function pointer table,
 * and "_vtbl_t" for its typedef.
 *
 * Interface Lifetime:
 *
 * The POSIX-like interface itself has a lifetime controlled by reference
 * counting.  When the reference count becomes zero, it should be released
 * by the appropriate interface method, and thereafter not used at all.
 *
 * But many related interfaces follow the single-owner idiom, where at any
 * given time there is exactly one "owner" who decides the lifetime of the
 * interface or transfers that privilege to the next owner.  In most cases
 * a singly-owned reference must be disowned (typically destroyed and
 * deallocated) before the interface that provided it is destroyed.
 * (We use "disown" to avoid conflict with the C++ "delete" keyword
 * when a C++ program includes this header.)
 *
 * Sometimes a single-owner interface provides a way to replace that
 * interface with another, typically for the purpose of iteration.
 * In that case any limits on the lifetime of the original interface
 * are inherited by any replacement interface.  Also, on error any
 * output argument that would receive an interface pointer returns
 * NULL except where otherwise specified.
 *
 * No interface will ensure that the program itself is kept alive.
 *
 * Using Interfaces Together:
 *
 * An interface may supply interfaces to other objects.  Such subordinate
 * interfaces should not be used in combination with another top-level
 * interface unless that is explicitly allowed by the documentation.
 *
 * If a subordinate interface provides lifetime controls, then it will keep
 * whatever it needs of the top-level *implementation* alive even after the
 * top-level *interface* is released (except as otherwise documented).
 *
 * Thread Safety and Reentrancy:
 *
 * Interface calls are designed to be thread-safe and reentrant,
 * except as explicitly documented otherwise, and except that you
 * must synchronize concurrent calls to any interface that uses the
 * single-owner idiom (and thus transfer ownership between threads).
 *
 * Be assured that any undocumented implementation details shared between
 * different single-owner interfaces will be synchronized automatically.
 * For example, if two single-owner interface instances happen to share
 * some state data between their implementations, data not contemplated
 * by the interface as such, then those implementations must automatically
 * synchronize access to the shared data without help from interface clients.
 *
 * As an exception to some of the above rules, clients may safely synchronize
 * on singly-owned synchronization objects without owning them.  (Otherwise
 * what would be the point of a synchronization object?)  However, deletion
 * of a synchronization object requires that all threads stop synchronizing
 * on it before the final owner begins deletion.
 *
 * Except as otherwise documented, reference-counted interfaces can be safely
 * shared between threads with no client synchronization, but adjustments to
 * reference counts must be atomic, and only the thread that drops that count
 * to zero should release the interface.
 *
 * Where the lifetime of an interface is dictated by context, the thread safety
 * of that interface is also dictated by that context.  Check documentation.
 *
 * Where POSIX distinguishes a legacy call from a thread-safe or reentrant
 * call, it generally adds "_r" to the name.  When we reflect such cases we
 * base our call only on the safer variant, and drop the "_r".
 *
 * Error Numbers:
 *
 * Instead of providing a thread-specific last error number in "errno", calls
 * that might error out simply return an error number, or 0 on success.  As
 * in POSIX, the type of such error numbers is the C language "int" type.
 *
 * Any interface operation that returns "int" and does not otherwise
 * document the returned value should be returning an error number.
 *
 * Unfortunately we have no way to provide pthread_once functionality
 * or to support the static initialization of synchronication objects.
 *
 * Environment Variables:
 *
 * In order to avoid thread-safety issues this interface does NOT provide
 * any access to the environment, much less any way to modify it.  Instead
 * the initial environment is typically supplied along with this interface.
 *
 * Locale:
 *
 * Full locale support would involve many thread-safety and portability issues.
 * Rather than resolve these issues now, the current version of this interface
 * provides only the behavior that you would get by calling
 *
 *     setlocale(LC_ALL, "");
 *
 * at the start of main() and then leaving the locale unchanged thereafter,
 * except that the encoding is always UTF-8 regardless of other locale choices.
 *
 * Timezone:
 *
 * Full thread-safe timezone support could be tricky.  For now
 * we provide only the behavior that you would get by calling
 *
 *     tzset()
 *
 * at the start of main() and then leaving the timezone unchanged thereafter.
 *
 * Object Orientation:
 *
 * One way in which our form departs from that of POSIX, even where
 * our substance does not, is that we prefer an object-oriented style.
 * We use object-oriented polymorphism not only for encapsulation,
 * but also to shorten the operation names a bit.  For example,
 * "setstacksize" instead of "thread_attr_setstacksize".
 *
 * But there should be few surprises to those familiar with object-oriented
 * languages; we intend a straightforward object-oriented binding of POSIX
 * functionality.  Still, there are a few novelties, as noted in comments.
 *
 * An interface is a struct containing a "vtbl" member that points to
 * a "virtual function table" similar to what a C++ compiler might generate,
 * though we allow it to contain constant values, not just method pointers.
 * Virtual function tables end in special pointers that help to detect errors
 * in their initialization, and also to serve as documentation of where one
 * interface ends and an extension begins.
 *
 * Interfaces may be extended by creating an extension struct and a virtual
 * function table that may be used as if they were the original.  You would
 * type cast in order to convert between the two interface types.
 *
 * But interfaces may also include other interfaces at nonzero byte offsets,
 * in a manner similar to what a C++ compiler might generate in the context of
 * multiple inheritance.  The included interface is extended, but conversion
 * requires pointer adjustment.  (Do NOT adjust NULL pointers.)
 *
 * Interfaces can be implemented by elaborating on one of the two extension
 * techniques mentioned above.
 *
 * Operation call:
 *
 * To call a operation, first fetch the method address from
 * the struct pointed to by the "vtbl" member of the interface.
 *
 * For most operations, pass a non-NULL pointer to the interface as
 * the first argument.
 *
 * But when calling an operation that may consume one reference to
 * its interface, instead pass a non-NULL pointer to such a pointer:
 * the designated interface pointer will be set to NULL if and only
 * if the reference is consumed.
 *
 * Method modification:
 *
 * Be sure that interface clients never modify "vtbl" members.  (We would
 * declare them "const" if that did not interfere with implementation code.)
 *
 * WARNING: But we DO allow *implementations* to modify "vtbl" members, and not
 * just during construction and destruction.  This flexibility goes beyond what
 * you see in C++, and is one of the few "novelties" we mentioned earlier.
 * Thus an interface client must NOT cache method pointers except where that is
 * allowed either explicitly or as a consequence of some other guarantee.  For
 * example, you may cache the method pointers of a thread-safe interface, and
 * a "const" operation must not modify "vtbl" because it is observable state.
 *
 * The above policy provides some opportunities for implementation optimization
 * at the cost of some opportunities for client optimization.  Our rationale is
 * that typically implementations are fewer and more widely-used than clients,
 * and therefore it is best to spend optimization effort on implementations.
 *
 * Generally the virtual function tables themselves should not be modified
 * because they may be shared among objects.  If you go against this policy
 * in any observable way, then be sure to document the oddity very clearly.
 *
 * Optional Values:
 *
 * Wherever practical, optional values are expressed by pointers in the
 * usual way: non-NULL means a value, and NULL means no value.  The caller
 * may be asked to supply storage for the value, as in POSIX readdir_r.
 *
 * Allocator Interfaces:
 *
 * A number of POSIX calls take a memory buffer as an argument
 * and have various ways of avoiding or handling the case in which
 * the given buffer is too small to hold some variably-sized result.
 *
 * But in this POSIX-like interface, we typically depart from that
 * approach in one of two ways.  The first is to pass a pointer to
 * a vrpsx_allocator_t that specifies how the caller wishes memory
 * to be allocated for the result.  A system allocator is provided so
 * that the caller need not implement vrpsx_allocator_t unless desired.
 *
 * The second way in which we avoid the passed-in-buffer approach is to return
 * an interface to the result.  That interface breaks down the result into
 * smaller pieces in some way, and its lifetime controls whatever internal
 * allocations are necessary in order to maintain the result after the
 * interface call returns.  Be sure to see the specific documentation
 * for the usable lifetime of the returned interface.
 *
 * Intentional Omissions:
 *
 * We intentionally omit stand-alone C library functions such as strlen()
 * that need not be standardized for compatibility with the implementation
 * or with other clients of the same interface.
 *
 * Extensions:
 *
 * There are a few extensions to POSIX functionality in order to support some
 * features that are widely available in various kinds of UNIX, BSD, and Linux,
 * even though POSIX does not define a standard interface for them.  Though
 * the enosys error is possible in general, it is more likely in such cases,
 * and clients should beware.
 *
 * Alternatives:
 *
 * The libposix project also aims to promote cross-platform compatibility:
 *
 *     http://libposix.sourceforge.net/
 *
 * We expect that libposix will require much less porting for existing
 * programs that were written for POSIX environments, because it aims to
 * faithfully implement the POSIX 2008 standard.  It may also be faster
 * because it replaces standard libraries instead of layering on top.
 *
 * By contrast, this interface breaks source-level POSIX compatibility in
 * order to achieve a high binary-level compatibility with a wide array of
 * execution environments (though you must still match the instruction set
 * and C language calling convention).  You should be able to implement
 * this interface relatively quickly on top of many operating systems,
 * browsers, and even applications.  Furthermore, such environments can
 * support multiple versions of this interface simultaneously, selectively
 * sandbox them, instrument them, etc.
 *
 * So it may be that there is a place for both of these interfaces.  In fact,
 * you should have no trouble implementing this interface on top of libposix
 * once that project implements the needed functionality.
 */

#ifndef VRPSX_SYSTEM_H
#define VRPSX_SYSTEM_H 1

#include "vrpsx_allocator.h"
#include "vrpsx_system_error.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/* Indicates a contiguous substring of a possibly-larger string.
 */
typedef struct vrpsx_substring_struct {
    /* The beginning of the substring.
     */
    char const *begin;

    /* The number of bytes in the substring (not necessarily the
     * number of characters in the case of multibyte encodings).
     *
     * Typically does not count any terminating NUL that might
     * or might not be present in the containing string.
     */
    size_t size;
} vrpsx_substring_t;


/* An interface to an object describing the system at the moment queried.
 *
 * Interface Lifetime: single-owner.
 * Thread Safety: requires synchronization.
 */
typedef struct vrpsx_utsname_struct {
    struct vrpsx_utsname_vtbl_struct const *vtbl;
} vrpsx_utsname_t;

typedef struct vrpsx_utsname_end_struct {
} vrpsx_utsname_end_t;

typedef struct vrpsx_utsname_vtbl_struct {
    /* Destroys and deallocates this interface (and the underlying object).
     * As appropriate, deallocation occurs even if destruction fails.
     * Clears the given pointer variable.
     */
    void (*disown)(
        vrpsx_utsname_t **utsname);

    char const *(*machine)(
        vrpsx_utsname_t const *utsname);

    char const *(*nodename)(
        vrpsx_utsname_t const *utsname);

    char const *(*release)(
        vrpsx_utsname_t const *utsname);

    char const *(*sysname)(
        vrpsx_utsname_t const *utsname);

    char const *(*version)(
        vrpsx_utsname_t const *utsname);

    /* This operation is an extension to POSIX that is specific
     * to the issue of virtualization of a POSIX-like interface.
     *
     * Though the virtual POSIX-like interface itself has a version, there
     * may be multiple implementations of that interface even on a particular
     * version of a particular platform.  An interface client may wish to work
     * around bugs or incomplete features in a given implementation of this
     * interface, as distinct from any issues associated with the POSIX-like
     * system itself, to which the interface connects.  A client may use this
     * version string to try to detect such issues and work around them.
     *
     * The returned pointer must not be NULL and must point to a NUL-terminated
     * string.  There are no other restrictions on the text; an implementation
     * may even return the empty string.  In that case, and when the value is
     * not recognized by the client, the client should assume that the version
     * of the implementation is more recent than any known at the time it was
     * written, and treat it generically.  In particularly, it should disable
     * any workarounds that are incompatible with a non-buggy implementation.
     * Of course, in that case the client cannot be expected to work around
     * any bugs or incomplete features beyond generic responses to "enosys".
     */
    char const *(*virtualization)(
        vrpsx_utsname_t const *utsname);

    /* End marker for documentation and to help detect initialization bugs.
     */
    vrpsx_utsname_end_t const *utsname_end;
} vrpsx_utsname_vtbl_t;


typedef vrpsx_int64_t vrpsx_time_t;

typedef struct vrpsx_timespec_struct {
    vrpsx_time_t tv_sec;
    long tv_nsec;
} vrpsx_timespec_t;

typedef struct vrpsx_tm_struct {
    int tm_sec;
    int tm_min;
    int tm_hour;
    int tm_mday;
    int tm_mon;
    int tm_year;
    int tm_wday;
    int tm_yday;
    int tm_isdst;
} vrpsx_tm_t;

typedef vrpsx_int64_t vrpsx_clockid_t;


typedef vrpsx_int64_t vrpsx_id_t;

typedef vrpsx_int64_t vrpsx_uid_t;

typedef vrpsx_int64_t vrpsx_gid_t;

typedef vrpsx_int64_t vrpsx_pid_t;


/* An interface to an object describing a user.
 *
 * Interface Lifetime: single-owner.
 * Thread Safety: requires synchronization.
 */
typedef struct vrpsx_passwd_struct {
    struct vrpsx_passwd_vtbl_struct const *vtbl;
} vrpsx_passwd_t;

typedef struct vrpsx_passwd_end_struct {
} vrpsx_passwd_end_t;

typedef struct vrpsx_passwd_vtbl_struct {
    /* Destroys and deallocates this interface (and the underlying object).
     * As appropriate, deallocation occurs even if destruction fails.
     * Clears the given pointer variable.
     */
    void (*disown)(
        vrpsx_passwd_t **passwd);

    char const *(*dir)(
        vrpsx_passwd_t const *passwd);

    char const *(*name)(
        vrpsx_passwd_t const *passwd);

    char const *(*shell)(
        vrpsx_passwd_t const *passwd);

    vrpsx_gid_t (*gid)(
        vrpsx_passwd_t const *passwd);

    vrpsx_uid_t (*uid)(
        vrpsx_passwd_t const *passwd);

    /* End marker for documentation and to help detect initialization bugs.
     */
    vrpsx_passwd_end_t const *passwd_end;
} vrpsx_passwd_vtbl_t;


typedef struct vrpsx_system_struct vrpsx_system_t;


/* An interface associated with a particular vrpsx_system_t interface.
 *
 * Most interfaces whose methods may fail with an vrpsx_errno_t value
 * should extend this interface in order to make it easy to compose an
 * error message string without access to a separately tracked pointer
 * to the correct vrpsx_system_t interface.
 *
 * WARNING: Often the vrpsx_system_t interface pointer maintained
 * by the implementation of this interface does NOT own a reference
 * to that vrpsx_system_t.  Rather, most interfaces associated with
 * a vrpsx_system_t assume that YOU will keep that vrpsx_system_t
 * alive until you are no longer using the associated interface.
 *
 * In particular, an interface such as vrpsx_sigset_t can disclaim any
 * need to be destroyed and still be a subinterface of this interface.
 *
 * System Reference On Error:
 *
 * Some calls into extensions of this interface may both destroy that
 * interface and return a nonzero error number.  But then how would
 * the caller obtain the correct vrpsx_system_t interface, either to
 * test for particular error constants or to obtain error text?
 * To address this scenario, we adopt the following policy.
 *
 * Every method call of an extension of this interface that destroys
 * that interface AND also returns a nonzero error number must also
 * grant the caller a counted reference to a vrpsx_system_t interface
 * that it may use to handle that error.
 *
 * More specifically, an extension interface method with the potential
 * for such behavior must take an argument of type (vrpsx_system_t **)
 * with a comment mentioning "System Reference On Error".
 *
 * If that argument has a non-NULL value, then the method must overwrite
 * the prior value of the indicated (vrpsx_system_t *) variable with:
 *
 *   - NULL if the method returns 0 for the error number.
 *
 *   - A counted reference to the vrpsx_system_t interface that defines
 *     the error, if the method returns a nonzero error number.  NOTE:
 *     In this case, the caller must eventually release that reference.
 *
 * Interface Lifetime: dictated by subinterface.
 * Thread Safety: thread-safe.
 * Async-signal-safe: yes.
 */
typedef struct vrpsx_system_associate_struct {
#define VRPSX_SYSTEM_ASSOCIATE_MEMBERS(VTBL)    \
    VTBL const *vtbl;

    VRPSX_SYSTEM_ASSOCIATE_MEMBERS(
        struct vrpsx_system_associate_vtbl_struct)
} vrpsx_system_associate_t;

typedef struct vrpsx_system_associate_end_struct {
} vrpsx_system_associate_end_t;

typedef struct vrpsx_system_associate_vtbl_struct {
#define VRPSX_SYSTEM_ASSOCIATE_VTBL_MEMBERS(TYPE)                       \
    /* Returns a pointer to the associated vrpsx_system_t interface.    \
     * The reference count of the returned interface remains unchanged. \
     */                                                                 \
    vrpsx_system_t *(*get_system)(                                      \
        TYPE const *associate);                                         \
                                                                        \
    /* End marker for documentation and to help detect initialization bugs. \
     */                                                                 \
    vrpsx_system_associate_end_t const *system_associate_end;

    VRPSX_SYSTEM_ASSOCIATE_VTBL_MEMBERS(vrpsx_system_associate_t);
} vrpsx_system_associate_vtbl_t;


/* It seems unlikely that even future binary-incompatible variants
 * of this interface would need to depart from "int", but by using
 * this typedef we provide convenient documentation of intent.
 */
typedef int vrpsx_signo_t;

/* An interface to a signal set object.
 *
 * Extends: vrpsx_system_associate_t.
 * Interface Lifetime: dictated by context.
 * Thread Safety: dictated by context (typically requires synchronization).
 * Async-signal-safe: yes.
 */
typedef struct vrpsx_sigset_struct {
    VRPSX_SYSTEM_ASSOCIATE_MEMBERS(struct vrpsx_sigset_vtbl_struct)
} vrpsx_sigset_t;

typedef struct vrpsx_sigset_end_struct {
} vrpsx_sigset_end_t;

typedef struct vrpsx_sigset_vtbl_struct {
    VRPSX_SYSTEM_ASSOCIATE_VTBL_MEMBERS(vrpsx_sigset_t)

    /* Assigns the signal set of *source to that of this interface.
     * State associated with extension interfaces is not copied.
     * The two interfaces must have originated in the same vrpsx_system_t.
     */
    void (*assign_sigset)(
        vrpsx_sigset_t *set,
        vrpsx_sigset_t const *source);

    /* Swaps signal sets with the given interface.
     * State associated with extension interfaces is not swapped.
     * The two interfaces must have originated in the same vrpsx_system_t.
     */
    void (*swap_sigset)(
        vrpsx_sigset_t *set,
        vrpsx_sigset_t *other);

    vrpsx_errno_t (*empty)(
        vrpsx_sigset_t *set);

    vrpsx_errno_t (*fill)(
        vrpsx_sigset_t *set);

    vrpsx_errno_t (*del)(
        vrpsx_sigset_t *set,
        vrpsx_signo_t signo);

    vrpsx_errno_t (*add)(
        vrpsx_sigset_t *set,
        vrpsx_signo_t signo);

    /* Returns an error number; on failure the
     * value of *out_is_member is unspecified.
     */
    vrpsx_errno_t (*ismember)(
        vrpsx_sigset_t const *set,
        vrpsx_signo_t signo,
        int *out_is_member);

    /* End marker for documentation and to help detect initialization bugs.
     */
    vrpsx_sigset_end_t const *sigset_end;
} vrpsx_sigset_vtbl_t;


/* This type is a placeholder for forward compatibility.  As yet we
 * neither define this type nor provide any ways to manipulate it.
 *
 * 1. An extension of this interface might define this type and translate
 * implementation data into this type before calling a signal handler.  This
 * approach keeps things simple for the interface client; in particular, it
 * should be able to add a "sigwaitinfo" operation that uses the same type.
 *
 * 2. Alternatively, an extension might leave this type undefined and use
 * it as an opaque handle to implementation data; signal handlers would
 * then have to use other interface operations to interrogate those data.
 * This approach might make implementation easier on POSIX-like systems,
 * and lazy translation of signal information might be very slightly faster.
 *
 * We leave both options open at this point.
 */
typedef struct vrpsx_siginfo_struct vrpsx_siginfo_t;

typedef void vrpsx_sa_handler_t(int);

typedef void vrpsx_sa_sigaction_t(int, vrpsx_siginfo_t *, void *);

typedef int vrpsx_sa_flags_t;

/* An interface to a signal action object.
 *
 * Extends: vrpsx_system_associate_t.
 * Interface Lifetime: dictated by context.
 * Thread Safety: dictated by context (typically requires synchronization).
 * Async-signal-safe: yes.
 */
typedef struct vrpsx_sigaction_struct {
    VRPSX_SYSTEM_ASSOCIATE_MEMBERS(struct vrpsx_sigaction_vtbl_struct)
} vrpsx_sigaction_t;

typedef struct vrpsx_sigaction_end_struct {
} vrpsx_sigaction_end_t;

typedef struct vrpsx_sigaction_vtbl_struct {
    VRPSX_SYSTEM_ASSOCIATE_VTBL_MEMBERS(vrpsx_sigaction_t)

    /* Assigns the vrpsx_sigaction_t data of *source to that of this
     * interface.  State associated with extension interfaces is not copied.
     * The two interfaces must have originated in the same vrpsx_system_t.
     */
    void (*assign_sigaction)(
        vrpsx_sigaction_t *action,
        vrpsx_sigaction_t const *source);

    /* Swaps vrpsx_sigaction_t data with the given interface.
     * State associated with extension interfaces is not swapped.
     * The two interfaces must have originated in the same vrpsx_system_t.
     */
    void (*swap_sigaction)(
        vrpsx_sigaction_t *action,
        vrpsx_sigaction_t *other);

    /* Returns NULL if the sa_siginfo flag is set.
     */
    vrpsx_sa_handler_t *(*get_handler)(
        vrpsx_sigaction_t const *action);

    /* Clears the sa_siginfo flag.
     */
    void (*set_handler)(
        vrpsx_sigaction_t *action,
        vrpsx_sa_handler_t *handler);

    /* Returns NULL if the sa_siginfo flag is clear.
     */
    vrpsx_sa_sigaction_t *(*get_sigaction)(
        vrpsx_sigaction_t const *action);

    /* Sets the sa_siginfo flag.
     */
    void (*set_sigaction)(
        vrpsx_sigaction_t *action,
        vrpsx_sa_sigaction_t *handler);

    /* Returns a non-NULL non-modifiable interface to the
     * current set of signals to block during handler execution.
     */
    vrpsx_sigset_t const *(*get_mask_const)(
        vrpsx_sigaction_t const *action);

    /* Returns a non-NULL modifiable interface to the
     * current set of signals to block during handler execution.
     */
    vrpsx_sigset_t *(*get_mask)(
        vrpsx_sigaction_t *action);

    vrpsx_sa_flags_t (*get_flags)(
        vrpsx_sigaction_t const *action);

    /* Leaves the sa_siginfo flag unchanged.
     *
     * Sets all supported flags, which in theory should be all flags
     * specified by this interface but in practice might fall short.
     *
     * If any flags could not be set then this operation returns enosys.
     * No other failures are possible.
     */
    vrpsx_errno_t (*set_flags)(
        vrpsx_sigaction_t *action,
        vrpsx_sa_flags_t flags);

    /* End marker for documentation and to help detect initialization bugs.
     */
    vrpsx_sigaction_end_t const *sigaction_end;
} vrpsx_sigaction_vtbl_t;


/* We still require the thread ID type to be an integer type, despite
 * recent changes to the POSIX standard that drop that requirement.
 *
 * Perhaps a future version of this interface will make this type more opaque,
 * but there are issues related to encapsulating the size of pthread_t.
 * We might wish to replace the notion of "thread ID" with "thread interface".
 *
 * Meanwhile, if your thread implementation does not use an integer thread ID,
 * then you may need to find some way to assign integer IDs and map them.
 */
typedef vrpsx_int64_t vrpsx_thread_t;


typedef void *vrpsx_thread_function_t(void *);


/* An interface to a thread attributes object.
 *
 * Extends: vrpsx_system_associate_t.
 * Interface Lifetime: single-owner.
 * Thread Safety: requires synchronization.
 */
typedef struct vrpsx_thread_attr_struct {
    VRPSX_SYSTEM_ASSOCIATE_MEMBERS(struct vrpsx_thread_attr_vtbl_struct)
} vrpsx_thread_attr_t;

typedef struct vrpsx_thread_attr_end_struct {
} vrpsx_thread_attr_end_t;

typedef struct vrpsx_thread_attr_vtbl_struct {
    VRPSX_SYSTEM_ASSOCIATE_VTBL_MEMBERS(vrpsx_thread_attr_t)

    /* Destroys and deallocates this interface.
     * As appropriate, deallocation occurs even if destruction fails.
     * Clears the given pointer variable.
     */
    vrpsx_errno_t (*disown)(
        vrpsx_thread_attr_t **attr,
        vrpsx_system_t **sroe); /* see "System Reference On Error" */

    /* Like POSIX "pthread_create", but adjusted
     * to be a method call on thread attributes.
     */
    vrpsx_errno_t (*create)(
        vrpsx_thread_attr_t const *attr,
        vrpsx_thread_t *thread,
        vrpsx_thread_function_t *function,
        void *argument);

    vrpsx_errno_t (*getdetachstate)(
        vrpsx_thread_attr_t const *attr,
        int *detach_state);

    vrpsx_errno_t (*setdetachstate)(
        vrpsx_thread_attr_t *attr,
        int detach_state);

    vrpsx_errno_t (*getscope)(
        vrpsx_thread_attr_t const *attr,
        int *contention_scope);

    vrpsx_errno_t (*setscope)(
        vrpsx_thread_attr_t *attr,
        int contention_scope);

    vrpsx_errno_t (*getstacksize)(
        vrpsx_thread_attr_t const *attr,
        size_t *stack_size);

    vrpsx_errno_t (*setstacksize)(
        vrpsx_thread_attr_t *attr,
        size_t stack_size);

    /* End marker for documentation and to help detect initialization bugs.
     */
    vrpsx_thread_attr_end_t const *thread_attr_end;
} vrpsx_thread_attr_vtbl_t;


typedef void vrpsx_thread_key_destructor_t(void *);


/* An interface to key for thread-specific storage.
 *
 * Extends: vrpsx_system_associate_t.
 *
 * Interface Lifetime: until disowned.
 *
 * Thread Safety: creation and destruction must be synchronized,
 *                but usage is thread-specific.
 */
typedef struct vrpsx_thread_key_struct {
    VRPSX_SYSTEM_ASSOCIATE_MEMBERS(struct vrpsx_thread_key_vtbl_struct)
} vrpsx_thread_key_t;

typedef struct vrpsx_thread_key_end_struct {
} vrpsx_thread_key_end_t;

typedef struct vrpsx_thread_key_vtbl_struct {
    VRPSX_SYSTEM_ASSOCIATE_VTBL_MEMBERS(vrpsx_thread_key_t)

    /* Destroys and deallocates this interface and the underlying key.
     * As appropriate, deallocation occurs even if destruction fails.
     * Clears the given pointer variable.
     *
     * WARNING: As in POSIX, you must be sure that all threads
     * have stopped using the key before you destroy it.
     */
    vrpsx_errno_t (*disown)(
        vrpsx_thread_key_t **key,
        vrpsx_system_t **sroe); /* see "System Reference On Error" */

    void *(*get)(
        vrpsx_thread_key_t const *key);

    vrpsx_errno_t (*set)(
        vrpsx_thread_key_t *key,
        void const *value);

    /* End marker for documentation and to help detect initialization bugs.
     */
    vrpsx_thread_key_end_t const *thread_key_end;
} vrpsx_thread_key_vtbl_t;


/* An interface to a mutex.
 *
 * Extends: vrpsx_system_associate_t.
 *
 * Interface Lifetime: single-owner.
 *
 * Thread Safety: creation and destruction must be synchronized,
 *                but usage is thread-safe.
 */
typedef struct vrpsx_mutex_struct {
    VRPSX_SYSTEM_ASSOCIATE_MEMBERS(struct vrpsx_mutex_vtbl_struct)
} vrpsx_mutex_t;

typedef struct vrpsx_mutex_end_struct {
} vrpsx_mutex_end_t;

typedef struct vrpsx_mutex_vtbl_struct {
    VRPSX_SYSTEM_ASSOCIATE_VTBL_MEMBERS(vrpsx_mutex_t)

    /* Destroys and deallocates this interface.
     * As appropriate, deallocation may occur even if destruction fails.
     * Clears the given pointer variable.
     *
     * WARNING: As in POSIX, you must be sure that all threads
     * have stopped using the mutex before you destroy it.
     */
    vrpsx_errno_t (*disown)(
        vrpsx_mutex_t **mutex,
        vrpsx_system_t **sroe); /* see "System Reference On Error" */

    vrpsx_errno_t (*unlock)(
        vrpsx_mutex_t *mutex);

    vrpsx_errno_t (*lock)(
        vrpsx_mutex_t *mutex);

    vrpsx_errno_t (*trylock)(
        vrpsx_mutex_t *mutex);

    /* End marker for documentation and to help detect initialization bugs.
     */
    vrpsx_mutex_end_t const *mutex_end;
} vrpsx_mutex_vtbl_t;


/* An interface to a mutex attributes object.
 *
 * Extends: vrpsx_system_associate_t.
 * Interface Lifetime: single-owner.
 * Thread Safety: requires synchronization.
 */
typedef struct vrpsx_mutex_attr_struct {
    VRPSX_SYSTEM_ASSOCIATE_MEMBERS(struct vrpsx_mutex_attr_vtbl_struct)
} vrpsx_mutex_attr_t;

typedef struct vrpsx_mutex_attr_end_struct {
} vrpsx_mutex_attr_end_t;

typedef struct vrpsx_mutex_attr_vtbl_struct {
    VRPSX_SYSTEM_ASSOCIATE_VTBL_MEMBERS(vrpsx_mutex_attr_t)

    /* Destroys and deallocates this interface.
     * As appropriate, deallocation occurs even if destruction fails.
     * Clears the given pointer variable.
     */
    vrpsx_errno_t (*disown)(
        vrpsx_mutex_attr_t **attr,
        vrpsx_system_t **sroe); /* see "System Reference On Error" */

    /* Allocates and constructs a new mutex object.
     *
     * If successful, sets *mutex to point to an interface
     * to the new object; otherwise sets *mutex to NULL.
     * On success, the caller takes responsibility for eventually
     * invoking the "disown" operation on the mutex.
     */
    vrpsx_errno_t (*create)(
        vrpsx_mutex_attr_t const *attr,
        vrpsx_mutex_t **mutex);

    vrpsx_errno_t (*gettype)(
        vrpsx_mutex_attr_t const *attr,
        int *type);

    vrpsx_errno_t (*settype)(
        vrpsx_mutex_attr_t *attr,
        int type);

    /* End marker for documentation and to help detect initialization bugs.
     */
    vrpsx_mutex_attr_end_t const *mutex_attr_end;
} vrpsx_mutex_attr_vtbl_t;


/* An interface to a read-write lock.
 *
 * Extends: vrpsx_system_associate_t.
 *
 * Interface Lifetime: single-owner.
 *
 * Thread Safety: creation and destruction must be synchronized,
 *                but usage is thread-safe.
 */
typedef struct vrpsx_rwlock_struct {
    VRPSX_SYSTEM_ASSOCIATE_MEMBERS(struct vrpsx_rwlock_vtbl_struct)
} vrpsx_rwlock_t;

typedef struct vrpsx_rwlock_end_struct {
} vrpsx_rwlock_end_t;

typedef struct vrpsx_rwlock_vtbl_struct {
    VRPSX_SYSTEM_ASSOCIATE_VTBL_MEMBERS(vrpsx_rwlock_t)

    /* Destroys and deallocates this interface.
     * As appropriate, deallocation may occur even if destruction fails.
     * Clears the given pointer variable.
     *
     * WARNING: As in POSIX, you must be sure that all threads
     * have stopped using the read-write lock before you destroy it.
     */
    vrpsx_errno_t (*disown)(
        vrpsx_rwlock_t **rwlock,
        vrpsx_system_t **sroe); /* see "System Reference On Error" */

    vrpsx_errno_t (*unlock)(
        vrpsx_rwlock_t *rwlock);

    vrpsx_errno_t (*rdlock)(
        vrpsx_rwlock_t *rwlock);

    vrpsx_errno_t (*wrlock)(
        vrpsx_rwlock_t *rwlock);

    vrpsx_errno_t (*tryrdlock)(
        vrpsx_rwlock_t *rwlock);

    vrpsx_errno_t (*trywrlock)(
        vrpsx_rwlock_t *rwlock);

    /* End marker for documentation and to help detect initialization bugs.
     */
    vrpsx_rwlock_end_t const *rwlock_end;
} vrpsx_rwlock_vtbl_t;


/* An interface to a read-write lock attributes object.
 *
 * This interface is a placeholder for forward compatibility;
 * the current version of this interface supports only default
 * attributes for condition variables.
 *
 * Extends: vrpsx_system_associate_t.
 * Interface Lifetime: single-owner.
 * Thread Safety: requires synchronization.
 */
typedef struct vrpsx_rwlock_attr_struct {
    VRPSX_SYSTEM_ASSOCIATE_MEMBERS(struct vrpsx_rwlock_attr_vtbl_struct)
} vrpsx_rwlock_attr_t;

typedef struct vrpsx_rwlock_attr_end_struct {
} vrpsx_rwlock_attr_end_t;

typedef struct vrpsx_rwlock_attr_vtbl_struct {
    VRPSX_SYSTEM_ASSOCIATE_VTBL_MEMBERS(vrpsx_rwlock_attr_t)

    /* Destroys and deallocates this interface.
     * As appropriate, deallocation occurs even if destruction fails.
     * Clears the given pointer variable.
     */
    vrpsx_errno_t (*disown)(
        vrpsx_rwlock_attr_t **attr,
        vrpsx_system_t **sroe); /* see "System Reference On Error" */

    /* Allocates and constructs a new read-write lock object.
     *
     * If successful, sets *rwlock to point to an interface
     * to the new object; otherwise sets *rwlock to NULL.
     * On success, the caller takes responsibility for eventually
     * invoking the "disown" operation on the read-write lock.
     */
    vrpsx_errno_t (*create)(
        vrpsx_rwlock_attr_t const *attr,
        vrpsx_rwlock_t **rwlock);

    /* End marker for documentation and to help detect initialization bugs.
     */
    vrpsx_rwlock_attr_end_t const *rwlock_attr_end;
} vrpsx_rwlock_attr_vtbl_t;


/* An interface to a condition variable.
 *
 * Extends: vrpsx_system_associate_t.
 *
 * Interface Lifetime: single-owner.
 *
 * Thread Safety: creation and destruction must be synchronized,
 *                but usage is thread-safe.
 */
typedef struct vrpsx_cond_struct {
    VRPSX_SYSTEM_ASSOCIATE_MEMBERS(struct vrpsx_cond_vtbl_struct)
} vrpsx_cond_t;

typedef struct vrpsx_cond_end_struct {
} vrpsx_cond_end_t;

typedef struct vrpsx_cond_vtbl_struct {
    VRPSX_SYSTEM_ASSOCIATE_VTBL_MEMBERS(vrpsx_cond_t)

    /* Destroys and deallocates this interface.
     * As appropriate, deallocation occurs even if destruction fails.
     * Clears the given pointer variable.
     *
     * WARNING: As in POSIX, you must be sure that all threads
     * have stopped using the condition variable before you destroy it.
     */
    vrpsx_errno_t (*disown)(
        vrpsx_cond_t **cond,
        vrpsx_system_t **sroe); /* see "System Reference On Error" */

    vrpsx_errno_t (*broadcast)(
        vrpsx_cond_t *cond);

    vrpsx_errno_t (*signal)(
        vrpsx_cond_t *cond);

    vrpsx_errno_t (*wait)(
        vrpsx_cond_t *cond,
        vrpsx_mutex_t *mutex);

    vrpsx_errno_t (*timedwait)(
        vrpsx_cond_t *cond,
        vrpsx_mutex_t *mutex,
        vrpsx_timespec_t const *absolute_time);

    /* End marker for documentation and to help detect initialization bugs.
     */
    vrpsx_cond_end_t const *cond_end;
} vrpsx_cond_vtbl_t;


/* An interface to a condition variable attributes object.
 *
 * This interface is a placeholder for forward compatibility;
 * the current version of this interface supports only default
 * attributes for condition variables.
 *
 * Extends: vrpsx_system_associate_t.
 * Interface Lifetime: single-owner.
 * Thread Safety: requires synchronization.
 */
typedef struct vrpsx_cond_attr_struct {
    VRPSX_SYSTEM_ASSOCIATE_MEMBERS(struct vrpsx_cond_attr_vtbl_struct)
} vrpsx_cond_attr_t;

typedef struct vrpsx_cond_attr_end_struct {
} vrpsx_cond_attr_end_t;

typedef struct vrpsx_cond_attr_vtbl_struct {
    VRPSX_SYSTEM_ASSOCIATE_VTBL_MEMBERS(vrpsx_cond_attr_t)

    /* Destroys and deallocates this interface.
     * As appropriate, deallocation occurs even if destruction fails.
     * Clears the given pointer variable.
     */
    vrpsx_errno_t (*disown)(
        vrpsx_cond_attr_t **attr,
        vrpsx_system_t **sroe); /* see "System Reference On Error" */

    /* Allocates and constructs a new condition variable object.
     *
     * If successful, sets *cond to point to an interface
     * to the new object; otherwise sets *cond to NULL.
     * On success, the caller takes responsibility for eventually
     * invoking the "disown" operation on the condition variable.
     */
    vrpsx_errno_t (*create)(
        vrpsx_cond_attr_t const *attr,
        vrpsx_cond_t **cond);

    /* End marker for documentation and to help detect initialization bugs.
     */
    vrpsx_cond_attr_end_t const *cond_attr_end;
} vrpsx_cond_attr_vtbl_t;


/* This type is wide enough to encode both "int" and pointer values;
 * therefore the implementation should not need a translation table.
 */
typedef vrpsx_intptr_t vrpsx_fd_t;

typedef vrpsx_int64_t vrpsx_off_t;

typedef vrpsx_int64_t vrpsx_blkcnt_t;

typedef vrpsx_int64_t vrpsx_blksize_t;

typedef vrpsx_uint64_t vrpsx_nlink_t;

typedef vrpsx_int64_t vrpsx_dev_t;

typedef vrpsx_uint64_t vrpsx_ino_t;

typedef vrpsx_uint64_t vrpsx_mode_t;


/* Timestamps include the fraction of a second, as in IEEE Std 1003.1-2008.
 */
typedef struct vrpsx_stat_struct {
    vrpsx_timespec_t st_atim;
    vrpsx_timespec_t st_ctim;
    vrpsx_timespec_t st_mtim;
    vrpsx_off_t st_size;
    vrpsx_blkcnt_t st_blocks;
    vrpsx_blksize_t st_blksize;
    vrpsx_nlink_t st_nlink;
    vrpsx_dev_t st_rdev;
    vrpsx_dev_t st_dev;
    vrpsx_ino_t st_ino;
    vrpsx_uid_t st_uid;
    vrpsx_gid_t st_gid;
    vrpsx_mode_t st_mode;
} vrpsx_stat_t;

typedef enum vrpsx_mode_bits_enum {
    vrpsx_S_ISUID = 04000,
    vrpsx_S_ISGID = 02000,
    vrpsx_S_ISVTX = 01000,
    vrpsx_S_IRWXU = 00700,
    vrpsx_S_IRUSR = 00400,
    vrpsx_S_IWUSR = 00200,
    vrpsx_S_IXUSR = 00100,
    vrpsx_S_IRWXG = 00070,
    vrpsx_S_IRGRP = 00040,
    vrpsx_S_IWGRP = 00020,
    vrpsx_S_IXGRP = 00010,
    vrpsx_S_IRWXO = 00007,
    vrpsx_S_IROTH = 00004,
    vrpsx_S_IWOTH = 00002,
    vrpsx_S_IXOTH = 00001
} vrpsx_mode_bits_t;


/* An interface to a temporary copy of an entry within a directory stream.
 *
 * Interface Lifetime: until next entry is fetched or stream is disowned.
 * Thread Safety: requires synchronization.
 */
typedef struct vrpsx_dirent_struct {
    struct vrpsx_dirent_vtbl_struct const *vtbl;
} vrpsx_dirent_t;

typedef struct vrpsx_dirent_end_struct {
} vrpsx_dirent_end_t;

typedef struct vrpsx_dirent_vtbl_struct {
    char const *(*name)(
        vrpsx_dirent_t const *dir);

    vrpsx_ino_t (*ino)(
        vrpsx_dirent_t const *dir);

    /* End marker for documentation and to help detect initialization bugs.
     */
    vrpsx_dirent_end_t const *dirent_end;
} vrpsx_dirent_vtbl_t;


/* An interface to a directory stream.
 *
 * Extends: vrpsx_system_associate_t.
 * Interface Lifetime: single-owner.
 * Thread Safety: requires synchronization.
 */
typedef struct vrpsx_directory_struct {
    VRPSX_SYSTEM_ASSOCIATE_MEMBERS(struct vrpsx_directory_vtbl_struct)
} vrpsx_directory_t;

typedef struct vrpsx_directory_end_struct {
} vrpsx_directory_end_t;

typedef struct vrpsx_directory_vtbl_struct {
    VRPSX_SYSTEM_ASSOCIATE_VTBL_MEMBERS(vrpsx_directory_t)

    /* Closes the stream, then destroys and deallocates this interface.
     * As appropriate, destruction/deallocation occur even if closure fails.
     * As appropriate, deallocation occurs even if destruction fails.
     * Clears the given pointer variable.
     */
    vrpsx_errno_t (*disown)(
        vrpsx_directory_t **dir,
        vrpsx_system_t **sroe); /* see "System Reference On Error" */

    /* Sets *out_fd to the file descriptor, or to -1 on error.
     */
    vrpsx_errno_t (*dirfd)(
        vrpsx_directory_t *dir,
        vrpsx_fd_t *out_fd);

    /* If this operation successfully reads another entry, then
     * it sets *next to an interface to that entry and returns 0.
     * The returned interface is valid only until the next call to an
     * operation that modifies the current position within the directory.
     *
     * If this operation successfully detects the end of the directory,
     * then it sets *next to NULL and returns 0.
     *
     * On failure, this operation sets *next to NULL
     * and returns a nonzero error number.
     */
    vrpsx_errno_t (*readdir)(
        vrpsx_directory_t *dir,
        vrpsx_dirent_t **next);

    /* We depart from POSIX by returning an error number.
     */
    vrpsx_errno_t (*rewinddir)(
        vrpsx_directory_t *dir);

    /* End marker for documentation and to help detect initialization bugs.
     */
    vrpsx_directory_end_t const *directory_end;
} vrpsx_directory_vtbl_t;


/* An mount option possibly followed by more of them.
 *
 * (Purists may note that this list is not fully lazy because you can tell
 * if the tail is empty by checking for NULL--without any function call.
 * However, this approach is simple and more familiar to C programmers.)
 *
 * Extends: vrpsx_system_associate_t.
 * Interface Lifetime: single-owner.
 * Thread Safety: requires synchronization.
 */
typedef struct vrpsx_mount_option_struct {
    VRPSX_SYSTEM_ASSOCIATE_MEMBERS(struct vrpsx_mount_option_vtbl_struct)
} vrpsx_mount_option_t;

typedef struct vrpsx_mount_option_end_struct {
} vrpsx_mount_option_end_t;

typedef struct vrpsx_mount_option_vtbl_struct {
    VRPSX_SYSTEM_ASSOCIATE_VTBL_MEMBERS(vrpsx_mount_option_t)

    /* Replaces this interface with an interface to the next mount option
     * or NULL if there are no more or an error occurs.  Sets (*option)
     * to NULL and sets (*tail) to the new interface or NULL, but in
     * such an order that if (option == tail) then the new interface,
     * if any, is stored in (*tail).  If tail == NULL, then all resources
     * still associated with mount option traversal are released.
     *
     * End of traversal (either natural or because tail == NULL)
     * is not itself an error.
     *
     * The old value of *tail is ignored.
     */
    vrpsx_errno_t (*next)(
        vrpsx_mount_option_t **option,
        vrpsx_mount_option_t **tail,
        vrpsx_system_t **sroe); /* see "System Reference On Error" */

    /* The option text: the text you would use with the "mount" program.
     * Each type of filesystem may define its own set of allowed options.
     *
     * The underlying string buffer is owned by the implementation of this
     * interface and may be invalidated by the next call to this operation
     * or the end of the lifetime of this interface, either by calling its
     * "next" operation or for any other reason.
     */
    vrpsx_errno_t (*text)(
        vrpsx_mount_option_t const *option,
        vrpsx_substring_t *text);

    /* End marker for documentation and to help detect initialization bugs.
     */
    vrpsx_mount_option_end_t const *mount_option_end;
} vrpsx_mount_option_vtbl_t;


/* A mount possibly followed by more of them.
 *
 * Though not a POSIX standard, most UNIX-like operating systems
 * have mounts with at least the attributes provided by this object.
 *
 * Note that the allowed types and options vary, and may depend
 * not only on the type of operating system but also upon which
 * filesystem software packages are currently installed.
 *
 * (Purists may note that this list is not fully lazy because you can tell
 * if the tail is empty by checking for NULL--without any function call.
 * However, this approach is simple and more familiar to C programmers.)
 *
 * Extends: vrpsx_system_associate_t.
 * Interface Lifetime: single-owner.
 * Thread Safety: requires synchronization.
 */
typedef struct vrpsx_mount_struct {
    VRPSX_SYSTEM_ASSOCIATE_MEMBERS(struct vrpsx_mount_vtbl_struct)
} vrpsx_mount_t;

typedef struct vrpsx_mount_end_struct {
} vrpsx_mount_end_t;

typedef struct vrpsx_mount_vtbl_struct {
    VRPSX_SYSTEM_ASSOCIATE_VTBL_MEMBERS(vrpsx_mount_t)

    /* Replaces this interface with an interface to the next mount or NULL
     * if there are no more or there is an error.  Sets (*mount) to NULL.
     * Sets (*mount) to NULL and sets (*tail) to the new interface or NULL,
     * but in such an order that if (mount == tail) then the new interface,
     * if any, is stored in (*tail).  If tail == NULL, then all resources
     * still associated with mount table traversal are released.
     *
     * End of traversal (either natural or because tail == NULL)
     * is not itself an error.
     *
     * The old value of *tail is ignored.
     */
    vrpsx_errno_t (*next)(
        vrpsx_mount_t **mount,
        vrpsx_mount_t **tail,
        vrpsx_system_t **sroe); /* see "System Reference On Error" */

    /* Filesystem device name: the pathname of a device or file
     * (as in the case of a loopback filesystem) that is to be mounted.
     * This is the device name in the form that would be expected by
     * the "mount" program in order to establish the mount.
     *
     * The given substring is valid until the next call
     * to a non-const operation other than "options".
     */
    vrpsx_errno_t (*device)(
        vrpsx_mount_t const *mount,
        vrpsx_substring_t *device);

    /* Mount point: the pathname for the mount point, where
     * the mounted filesystem will enter the global namespace.
     *
     * The given substring is valid until the next call
     * to a non-const operation other than "options".
     */
    vrpsx_errno_t (*root)(
        vrpsx_mount_t const *mount,
        vrpsx_substring_t *root);

    /* Filesystem type: a filesystem type name, such
     * as you would use with the "mount" program.
     *
     * The given substring is valid until the next call
     * to a non-const operation other than "options".
     */
    vrpsx_errno_t (*type)(
        vrpsx_mount_t const *mount,
        vrpsx_substring_t *type);

    /* Stores in *options an interface to the first in a list of mount
     * options of mount options, or NULL if there are zero options or
     * an error occurrs.
     *
     * Option text spelling should be compatible with "mount -o ...",
     * except that separate "mount" switches may be translated into
     * fictional options for "mount -o ...", and except for any special
     * case required for automounter configuration, though in that case
     * it must still be compatible with the "mount" operation of the
     * vrpsx_system_t interface.  Each type of filesystem and
     * automounter may define its own set of allowed options.
     *
     * In addition to the usual single-owner semantics, the
     * returned interface is implicitly disowned by the next call
     * to this operation or the destruction of this interface.
     */
    vrpsx_errno_t (*options)(
        vrpsx_mount_t *mount,
        vrpsx_mount_option_t **options);

    /* End marker for documentation and to help detect initialization bugs.
     */
    vrpsx_mount_end_t const *mount_end;
} vrpsx_mount_vtbl_t;


/* The top-level POSIX-like system interface.
 * See also the introductory comments at the top of this header.
 *
 * You can cast a pointer to this interface to (vrpsx_counted_identified_t *)
 * because this interface extends vrpsx_counted_identified_t.
 *
 * All instances of vrpsx_error_t provided by this interface
 * are actually instances of vrpsx_system_error_t, and you
 * may cast between the corresponding pointer types.
 *
 * Extends: vrpsx_counted_identified_t.
 * Extends at nonzero offset: vrpsx_allocator_t.
 * Interface Lifetime: reference counted.
 * Thread Safety: thread-safe.
 */
struct vrpsx_system_struct {
#define VRPSX_SYSTEM_MEMBERS(VTBL)                              \
    VRPSX_COUNTED_IDENTIFIED_MEMBERS(VTBL)                      \
                                                                \
    /* A thread-safe allocator that is suitably aligned for all \
     * C data types; vrpsx_system_t extends vrpsx_allocator_t   \
     * through this member.                                     \
     *                                                          \
     * You only need to use this allocator if you have no other \
     * allocator or are planning to pass memory blocks to other \
     * clients of this interface.                               \
     */                                                         \
    vrpsx_allocator_t allocator;

    VRPSX_SYSTEM_MEMBERS(struct vrpsx_system_vtbl_struct)
};

typedef struct vrpsx_system_operations_end_struct {
} vrpsx_system_operations_end_t;

typedef struct vrpsx_system_errno_end_struct {
} vrpsx_system_errno_end_t;

typedef struct vrpsx_system_signo_end_struct {
} vrpsx_system_signo_end_t;

typedef struct vrpsx_system_signal_end_struct {
} vrpsx_system_signal_end_t;

typedef struct vrpsx_system_thread_end_struct {
} vrpsx_system_thread_end_t;

typedef struct vrpsx_system_fcntl_end_struct {
} vrpsx_system_fcntl_end_t;

typedef struct vrpsx_system_stat_end_struct {
} vrpsx_system_stat_end_t;

typedef struct vrpsx_system_time_end_struct {
} vrpsx_system_time_end_t;

typedef struct vrpsx_system_limits_end_struct {
} vrpsx_system_limits_end_t;

typedef struct vrpsx_system_stdlib_end_struct {
} vrpsx_system_stdlib_end_t;

typedef struct vrpsx_system_sysconf_end_struct {
} vrpsx_system_sysconf_end_t;

typedef struct vrpsx_system_pathconf_end_struct {
} vrpsx_system_pathconf_end_t;

typedef struct vrpsx_system_end_struct {
} vrpsx_system_end_t;

typedef struct vrpsx_system_vtbl_struct {
#define VRPSX_SYSTEM_VTBL_MEMBERS(TYPE)                                 \
    VRPSX_COUNTED_IDENTIFIED_VTBL_MEMBERS(TYPE)                         \
                                                                        \
    /* This operation sets *out_error to a non-NULL value and grants    \
     * the caller ownership of one reference to that error inteface.    \
     *                                                                  \
     * If the implementation cannot to provide the correct text then    \
     * it may substitute a fall-back error object with different text   \
     * and possibly a different error number as long as it ensures      \
     * that *out_error != NULL.                                         \
     *                                                                  \
     * If POSIX strerror_l(ern, LC_GLOBAL_LOCALE) is defined by the     \
     * underlying system and would succeed, then the error interface    \
     * provided by this operation supplies the same text as would be    \
     * returned by such a call.                                         \
     *                                                                  \
     * The old value of *out_error is ignored.                          \
     */                                                                 \
    void (*error_acquire)(                                              \
        TYPE *system,                                                   \
        vrpsx_errno_t ern,                                              \
        vrpsx_system_error_t **out_error);                              \
                                                                        \
    /* Equivalent To:                                                   \
     *      system->allocator.vtbl                                      \
     *          -> free_memory(&system->allocator, memory)              \
     */                                                                 \
    void (*free_memory)(                                                \
        TYPE *system,                                                   \
        void *memory);                                                  \
                                                                        \
    /* This operation does not return an error number, but on failure   \
     * you may assume that the error number is system->vtbl->enomem.    \
     *                                                                  \
     * Equivalent To:                                                   \
     *      system->allocator.vtbl                                      \
     *          -> allocate_memory(&system->allocator, size)            \
     */                                                                 \
    void *(*allocate_memory)(                                           \
        TYPE *system,                                                   \
        size_t size);                                                   \
                                                                        \
    void (*exit)(                                                       \
        TYPE *system,                                                   \
        int code);                                                      \
                                                                        \
    /* Allocates and constructs an object describing the system at the  \
     * time this function is called.  If the system is virtual, then    \
     * it is the virtual system that is described, NOT the real one.    \
     *                                                                  \
     * If successful, sets *name to point to an interface;              \
     * otherwise sets *name to NULL.  On success, the caller            \
     * takes responsibility for eventually invoking                     \
     * the "disown" operation on the object.                            \
     *                                                                  \
     * Note that failure is possible,                                   \
     * including memory allocation failure.                             \
     *                                                                  \
     * The old value of *name is ignored.                               \
     */                                                                 \
    vrpsx_errno_t (*uname)(                                             \
        TYPE *system,                                                   \
        vrpsx_utsname_t **out_name);                                    \
                                                                        \
    /* The resulting value is stored in *out_value.  On error -1 is     \
     * stored in *out_value.  The old value of *out_value is ignored.   \
     * The returned value is an error code or 0 for success.            \
     */                                                                 \
    vrpsx_errno_t (*sysconf)(                                           \
        TYPE *system,                                                   \
        int name,                                                       \
        long *out_value);                                               \
                                                                        \
    /* The resulting value is stored in *out_value.  On error -1 is     \
     * stored in *out_value.  The old value of *out_value is ignored.   \
     * The returned value is an error code or 0 for success.            \
     */                                                                 \
    vrpsx_errno_t (*pathconf)(                                          \
        TYPE *system,                                                   \
        char const *path,                                               \
        int name,                                                       \
        long *out_value);                                               \
                                                                        \
    /* The resulting value is stored in *out_value.  On error -1 is     \
     * stored in *out_value.  The old value of *out_value is ignored.   \
     * The returned value is an error code or 0 for success.            \
     */                                                                 \
    vrpsx_errno_t (*fpathconf)(                                         \
        TYPE *system,                                                   \
        vrpsx_fd_t fd,                                                  \
        int name,                                                       \
        long *out_value);                                               \
                                                                        \
    /* We depart from POSIX by returning an error number and using      \
     * a different allocation strategy.  The given allocator interface  \
     * is used to allocate the directory string, and *out_cwd is set    \
     * to point to that string, or to NULL on any kind of failure.      \
     * Allocation failure results in enomem.                            \
     *                                                                  \
     * This operation is reentrant except that the allocator should     \
     * not change the current directory directly or indirectly, even    \
     * to the same value it had before.                                 \
     *                                                                  \
     * Thread Safety: the caller must avoid concurrently                \
     * changing the current directory.                                  \
     */                                                                 \
    vrpsx_errno_t (*getcwd)(                                            \
        TYPE *system,                                                   \
        vrpsx_allocator_t *allocator, /* singular allocation;           \
                                       * NUL-terminated allocation;     \
                                       * NULL means &system->allocator */ \
        char **out_cwd);                                                \
                                                                        \
    /* Thread Safety: this value does not change in the current         \
     * version of this interface, making this call thread-safe.         \
     */                                                                 \
    vrpsx_int64_t (*get_timezone)(                                      \
        TYPE *system);                                                  \
                                                                        \
    /* Slight deviation from POSIX: returned strings are const.         \
     *                                                                  \
     * Thread Safety: these values do not change in the current         \
     * version of this interface, making this call thread-safe.         \
     */                                                                 \
    char const *const *(*get_tzname)(                                   \
        TYPE *system);                                                  \
                                                                        \
    vrpsx_errno_t (*gmtime)(                                            \
        TYPE *system,                                                   \
        vrpsx_time_t in_time,                                           \
        vrpsx_tm_t *out_time);                                          \
                                                                        \
    /* Thread Safety: the current version of this interface does        \
     * not provide a way to modify the timezone, making this call       \
     * thread-safe as long as no other party changes the time zone.     \
     */                                                                 \
    vrpsx_errno_t (*localtime)(                                         \
        TYPE *system,                                                   \
        vrpsx_time_t in_time,                                           \
        vrpsx_tm_t *out_time);                                          \
                                                                        \
    /* Thread Safety: in the current version of this interface          \
     * the timezone is a constant, making this call thread-safe.        \
     */                                                                 \
    vrpsx_errno_t (*mktime)(                                            \
        TYPE *system,                                                   \
        vrpsx_tm_t *io_tm,                                              \
        vrpsx_time_t *out_time);                                        \
                                                                        \
    vrpsx_errno_t (*time)(                                              \
        TYPE *system,                                                   \
        vrpsx_time_t *out_time);                                        \
                                                                        \
    vrpsx_errno_t (*clock_gettime)(                                     \
        TYPE *system,                                                   \
        vrpsx_clockid_t clock,                                          \
        vrpsx_timespec_t *out_time);                                    \
                                                                        \
    vrpsx_errno_t (*nanosleep)(                                         \
        TYPE *system,                                                   \
        vrpsx_timespec_t const *duration,                               \
        vrpsx_timespec_t *remaining);                                   \
                                                                        \
    /* We depart from POSIX by returning an error number and using      \
     * a different allocation strategy.  The given allocator interface  \
     * is used to allocate the name string, and *out_name is set        \
     * to point to that string, or to NULL on any kind of failure.      \
     * Allocation failure results in enomem.                            \
     *                                                                  \
     * This operation is reentrant as long as the allocator does not    \
     * modify the user database, even to the same state it had before.  \
     *                                                                  \
     * Concurrent modification of the user database is NOT safe.        \
     */                                                                 \
    vrpsx_errno_t (*getlogin)(                                          \
        TYPE *system,                                                   \
        vrpsx_allocator_t *allocator, /* singular allocation;           \
                                       * NUL-terminated allocation;     \
                                       * NULL means &system->allocator */ \
        char **out_name);                                               \
                                                                        \
    /* We depart from POSIX by returning an interface to the user data  \
     * instead of a raw structure.  We set *result to NULL on failure.  \
     * The old value of *result is ignored.                             \
     *                                                                  \
     * The returned interface is not affected by changes to the         \
     * user database that occur after this operation returns.           \
     */                                                                 \
    vrpsx_errno_t (*getpwnam)(                                          \
        TYPE *system,                                                   \
        char const *name,                                               \
        vrpsx_passwd_t **result);                                       \
                                                                        \
    vrpsx_pid_t (*getpid)(                                              \
        TYPE *system);                                                  \
                                                                        \
    vrpsx_pid_t (*getpgrp)(                                             \
        TYPE *system);                                                  \
                                                                        \
    /* On success, *out_pgid is set to the process group ID of the      \
     * specified process, but on failure *outPgid is set to -1.         \
     */                                                                 \
    vrpsx_errno_t (*getpgid)(                                           \
        TYPE *system,                                                   \
        vrpsx_pid_t pid,                                                \
        vrpsx_pid_t *out_pgid);                                         \
                                                                        \
    vrpsx_errno_t (*kill)(                                              \
        TYPE *system,                                                   \
        vrpsx_pid_t pid,                                                \
        vrpsx_signo_t signo);                                           \
                                                                        \
    vrpsx_errno_t (*raise)(                                             \
        TYPE *system,                                                   \
        vrpsx_signo_t signo);                                           \
                                                                        \
    /* Returns the pointer that would be returned                       \
     * by POSIX strsignal(), except that a NULL return                  \
     * is prohibited in favor of generic fall-back text.                \
     *                                                                  \
     * We also depart from POSIX by requiring thread safety.            \
     * In particular, the return points to constant or thread-local     \
     * storage.  (However, we do NOT guarantee async-signal-safety.)    \
     */                                                                 \
    char const *(*strsignal)(                                           \
        TYPE *system,                                                   \
        vrpsx_signo_t signo);                                           \
                                                                        \
    /* Constructs a new, empty signal set object in the provided        \
     * storage space.                                                   \
     *                                                                  \
     * The caller must provide sizeof_sigset bytes of storage space     \
     * with alignment alignof_sigset (or a multiple thereof).  NOTE:    \
     * These storage requirements may EXCEED those of vrpsx_sigset_t.   \
     *                                                                  \
     * If successful, this operation sets *set to point to              \
     * the new interface (which might be at an offset from the          \
     * beginning of the storage space) and returns 0.  There is         \
     * no destruction operation, nor is there any need to destroy       \
     * the interface or object.                                         \
     *                                                                  \
     * On failure this operation sets *set to NULL and returns an       \
     * error number.                                                    \
     *                                                                  \
     * The old value of *set is ignored and overwritten,                \
     * as are the old contents of the provided storage space.           \
     *                                                                  \
     * Async-signal-safe.  (Design note: that requirement is            \
     * why we had to separate construction from allocation.)            \
     */                                                                 \
    vrpsx_errno_t (*sigset_init)(                                       \
        TYPE *system,                                                   \
        void *storage,                                                  \
        vrpsx_sigset_t **set);                                          \
                                                                        \
    /* Constructs a new signal action object in the provided storage    \
     * space.  Initially the flags are 0, the handlers are NULL, and    \
     * the signal set is empty.  You should fill in the desired         \
     * details before use.                                              \
     *                                                                  \
     * The caller must provide sizeof_sigaction bytes of storage space  \
     * with alignment alignof_sigaction (or a multiple thereof).  NOTE: \
     * These storage requirements may EXCEED those of vrpsx_sigaction_t. \
     *                                                                  \
     * If successful, this operation sets *action to point to           \
     * the new interface (which might be at an offset from the          \
     * beginning of the storage space) and returns 0.  There is         \
     * no destruction operation, nor is there any need to destroy       \
     * the interface or object.                                         \
     *                                                                  \
     * On failure this operation sets *action to NULL                   \
     * and returns an error number.                                     \
     *                                                                  \
     * The old value of *action is ignored and overwritten,             \
     * as are the old contents of the provided storage space.           \
     *                                                                  \
     * Async-signal-safe.  (Design note: that requirement is            \
     * why we had to separate construction from allocation.)            \
     */                                                                 \
    vrpsx_errno_t (*sigaction_init)(                                    \
        TYPE *system,                                                   \
        void *storage,                                                  \
        vrpsx_sigaction_t **action);                                    \
                                                                        \
    /* The two vrpsx_sigaction_t interfaces must have                   \
     * originated in this interface; passing foreign                    \
     * interfaces is a (possibly undetected) bug.                       \
     */                                                                 \
    vrpsx_errno_t (*sigaction)(                                         \
        TYPE *system,                                                   \
        vrpsx_signo_t signo,                                            \
        vrpsx_sigaction_t const *action,                                \
        vrpsx_sigaction_t *old_action);                                 \
                                                                        \
    /* The given vrpsx_sigset_t interface must have                     \
     * originated in this interface; passing a foreign                  \
     * interface is a (possibly undetected) bug.                        \
     */                                                                 \
    vrpsx_errno_t (*sigwait)(                                           \
        TYPE *system,                                                   \
        vrpsx_sigset_t const *set,                                      \
        vrpsx_signo_t *signo);                                          \
                                                                        \
    /* The two vrpsx_sigset_t interfaces must have                      \
     * originated in this interface; passing foreign                    \
     * interfaces is a (possibly undetected) bug.                       \
     *                                                                  \
     * Async-signal-safe.                                               \
     */                                                                 \
    vrpsx_errno_t (*thread_sigmask)(                                    \
        TYPE *system,                                                   \
        int how,                                                        \
        vrpsx_sigset_t const *change,                                   \
        vrpsx_sigset_t *old_set);                                       \
                                                                        \
    /* Allocates and constructs a new thread attributes object.         \
     *                                                                  \
     * If successful, sets *attr to point to an interface               \
     * to the new object; otherwise sets *attr to NULL.                 \
     * On success, the caller takes responsibility for eventually       \
     * invoking the "disown" operation on the attributes object.        \
     *                                                                  \
     * The old value of *attr is ignored.                               \
     */                                                                 \
    vrpsx_errno_t (*thread_attr_create)(                                \
        TYPE *system,                                                   \
        vrpsx_thread_attr_t **attr);                                    \
                                                                        \
    /* Like POSIX "pthread_create" with default attributes.             \
     *                                                                  \
     * To create a thread with non-default attributes, call             \
     * thread_attr_create, adjust the attributes object, and            \
     * then call its "create" method.                                   \
     */                                                                 \
    vrpsx_errno_t (*thread_create)(                                     \
        TYPE *system,                                                   \
        vrpsx_thread_t *thread,                                         \
        vrpsx_thread_function_t *function,                              \
        void *argument);                                                \
                                                                        \
    vrpsx_errno_t (*thread_detach)(                                     \
        TYPE *system,                                                   \
        vrpsx_thread_t thread);                                         \
                                                                        \
    vrpsx_errno_t (*thread_join)(                                       \
        TYPE *system,                                                   \
        vrpsx_thread_t thread,                                          \
        void **value);                                                  \
                                                                        \
    vrpsx_errno_t (*thread_kill)(                                       \
        TYPE *system,                                                   \
        vrpsx_thread_t thread,                                          \
        vrpsx_signo_t signo);                                           \
                                                                        \
    int (*thread_equal)(                                                \
        TYPE *system,                                                   \
        vrpsx_thread_t thread1,                                         \
        vrpsx_thread_t thread2);                                        \
                                                                        \
    vrpsx_thread_t (*thread_self)(                                      \
        TYPE *system);                                                  \
                                                                        \
    vrpsx_errno_t (*thread_key_create)(                                 \
        TYPE *system,                                                   \
        vrpsx_thread_key_t **key,                                       \
        vrpsx_thread_key_destructor_t *destructor);                     \
                                                                        \
    /* Allocates and constructs a new mutex attributes object.          \
     *                                                                  \
     * If successful, sets *attr to point to an interface               \
     * to the new object; otherwise sets *attr to NULL.                 \
     * On success, the caller takes responsibility for eventually       \
     * invoking the "disown" operation on the attributes object.        \
     *                                                                  \
     * The old value of *attr is ignored.                               \
     */                                                                 \
    vrpsx_errno_t (*mutex_attr_create)(                                 \
        TYPE *system,                                                   \
        vrpsx_mutex_attr_t **attr);                                     \
                                                                        \
    /* Allocates and constructs a new mutex with default attributes.    \
     *                                                                  \
     * To create a mutex with non-default attributes,                   \
     * call mutex_attr_create, adjust the attributes object,            \
     * and then call its "create" method.                               \
     *                                                                  \
     * If successful, sets *mutex to point to an interface              \
     * to the new object; otherwise sets *mutex to NULL.                \
     * On success, the caller takes responsibility for eventually       \
     * invoking the "disown" operation on the mutex object.             \
     *                                                                  \
     * The old value of *mutex is ignored.                              \
     */                                                                 \
    vrpsx_errno_t (*mutex_create)(                                      \
        TYPE *system,                                                   \
        vrpsx_mutex_t **mutex);                                         \
                                                                        \
    /* Allocates and constructs a new read-write lock attributes        \
     * object.                                                          \
     *                                                                  \
     * If successful, sets *attr to point to an interface               \
     * to the new object; otherwise sets *attr to NULL.                 \
     * On success, the caller takes responsibility for eventually       \
     * invoking the "disown" operation on the attributes object.        \
     *                                                                  \
     * The old value of *attr is ignored.                               \
     */                                                                 \
    vrpsx_errno_t (*rwlock_attr_create)(                                \
        TYPE *system,                                                   \
        vrpsx_rwlock_attr_t **attr);                                    \
                                                                        \
    /* Allocates and constructs a new read-write lock                   \
     * with default attributes.                                         \
     *                                                                  \
     * To create a read-write lock with non-default attributes,         \
     * call rwlock_attr_create, adjust the attributes object,           \
     * and then call its "create" method.                               \
     *                                                                  \
     * If successful, sets *rwlock to point to an interface             \
     * to the new object; otherwise sets *rwlock to NULL.               \
     * On success, the caller takes responsibility for eventually       \
     * invoking the "disown" operation on the rwlock object.            \
     *                                                                  \
     * The old value of *rwlock is ignored.                             \
     */                                                                 \
    vrpsx_errno_t (*rwlock_create)(                                     \
        TYPE *system,                                                   \
        vrpsx_rwlock_t **rwlock);                                       \
                                                                        \
    /* Allocates and constructs a new condition variable attributes     \
     * object.                                                          \
     *                                                                  \
     * If successful, sets *attr to point to an interface               \
     * to the new object; otherwise sets *attr to NULL.                 \
     * On success, the caller takes responsibility for eventually       \
     * invoking the "disown" operation on the attributes object.        \
     *                                                                  \
     * The old value of *attr is ignored.                               \
     */                                                                 \
    vrpsx_errno_t (*cond_attr_create)(                                  \
        TYPE *system,                                                   \
        vrpsx_cond_attr_t **attr);                                      \
                                                                        \
    /* Allocates and constructs a new condition variable                \
     * with default attributes.                                         \
     *                                                                  \
     * To create a condition variable with non-default attributes,      \
     * call cond_attr_create, adjust the attributes object,             \
     * and then call its "create" method.                               \
     *                                                                  \
     * If successful, sets *cond to point to an interface               \
     * to the new object; otherwise sets *cond to NULL.                 \
     * On success, the caller takes responsibility for eventually       \
     * invoking "disown" on the condition variable object.              \
     *                                                                  \
     * The old value of *cond is ignored.                               \
     */                                                                 \
    vrpsx_errno_t (*cond_create)(                                       \
        TYPE *system,                                                   \
        vrpsx_cond_t **cond);                                           \
                                                                        \
    /* If the permissions argument will be ignored,                     \
     * then just pass an arbitrary value for it.                        \
     *                                                                  \
     * On success the file descriptor is stored in *out_fd;             \
     * on failure this operation stores -1 there.                       \
     * The old value of *out_fd is ignored.                             \
     */                                                                 \
    vrpsx_errno_t (*open)(                                              \
        TYPE *system,                                                   \
        char const *path,                                               \
        int flags,                                                      \
        int permissions,                                                \
        vrpsx_fd_t *out_fd);                                            \
                                                                        \
    vrpsx_errno_t (*close)(                                             \
        TYPE *system,                                                   \
        vrpsx_fd_t fd);                                                 \
                                                                        \
    /* Attempts to read in_size bytes.                                  \
     *                                                                  \
     * On success the number of bytes read is stored in *out_size.      \
     *                                                                  \
     * On failure *out_size is set to 0.                                \
     */                                                                 \
    vrpsx_errno_t (*read)(                                              \
        TYPE *system,                                                   \
        vrpsx_fd_t fd,                                                  \
        void *buffer,                                                   \
        size_t in_size,                                                 \
        size_t *out_size); /* non-NULL */                               \
                                                                        \
    /* Attempts to write in_size bytes.                                 \
     *                                                                  \
     * If out_size != NULL:                                             \
     *                                                                  \
     *     On success the number of bytes written                       \
     *     is stored in *out_size.                                      \
     *                                                                  \
     *     On failure *out_size is set to 0.                            \
     */                                                                 \
    vrpsx_errno_t (*write)(                                             \
        TYPE *system,                                                   \
        vrpsx_fd_t fd,                                                  \
        void const *buffer,                                             \
        size_t in_size,                                                 \
        size_t *out_size);                                              \
                                                                        \
    /* Attempts to read in_size bytes starting at the given offset.     \
     *                                                                  \
     * On success the number of bytes read is stored in *out_size.      \
     *                                                                  \
     * On failure *out_size is set to 0.                                \
     */                                                                 \
    vrpsx_errno_t (*pread)(                                             \
        TYPE *system,                                                   \
        vrpsx_fd_t fd,                                                  \
        void *buffer,                                                   \
        size_t in_size,                                                 \
        size_t offset,                                                  \
        size_t *out_size); /* non-NULL */                               \
                                                                        \
    /* Attempts to write in_size bytes starting at the given offset.    \
     *                                                                  \
     * If out_size != NULL:                                             \
     *                                                                  \
     *     On success the number of bytes written                       \
     *     is stored in *out_size.                                      \
     *                                                                  \
     *     On failure *out_size is set to 0.                            \
     */                                                                 \
    vrpsx_errno_t (*pwrite)(                                            \
        TYPE *system,                                                   \
        vrpsx_fd_t fd,                                                  \
        void const *buffer,                                             \
        size_t in_size,                                                 \
        size_t offset,                                                  \
        size_t *out_size);                                              \
                                                                        \
    vrpsx_errno_t (*unlink)(                                            \
        TYPE *system,                                                   \
        char const *path);                                              \
                                                                        \
    vrpsx_errno_t (*link)(                                              \
        TYPE *system,                                                   \
        char const *existing,                                           \
        char const *new_name);                                          \
                                                                        \
    vrpsx_errno_t (*symlink)(                                           \
        TYPE *system,                                                   \
        char const *target,                                             \
        char const *symlink_name);                                      \
                                                                        \
    /* Unlike POSIX, the result is allocated in the specified way       \
     * and stored in *result, with a NUL terminator appended.           \
     * The length of the result, not including the added NUL,           \
     * is stored in *result_length.  On failure, *result is             \
     * set to NULL and *result_length is set to 0.                      \
     *                                                                  \
     * This operation is reentrant except that the allocator itself     \
     * must not modify the symbolic link that is being read.            \
     *                                                                  \
     * But concurrent modification of the symlink is supported in the   \
     * sense that either the old link target or the new link target     \
     * will be placed in *result, though timing determines which one.   \
     *                                                                  \
     * This operation may repeatedly free and then allocate in          \
     * response to changing symlink target lengths, though it           \
     * must ensure eventual termination in some fashion.                \
     */                                                                 \
    vrpsx_errno_t (*readlink)(                                          \
        TYPE *system,                                                   \
        char const *path,                                               \
        vrpsx_allocator_t *allocator, /* singular allocation;           \
                                       * NUL-terminated allocation;     \
                                       * NULL means &system->allocator */ \
        char **result,                                                  \
        size_t *result_length);                                         \
                                                                        \
    vrpsx_errno_t (*chmod)(                                             \
        TYPE *system,                                                   \
        char const *path,                                               \
        vrpsx_mode_t mode);                                             \
                                                                        \
    vrpsx_errno_t (*fchmod)(                                            \
        TYPE *system,                                                   \
        vrpsx_fd_t fd,                                                  \
        vrpsx_mode_t mode);                                             \
                                                                        \
    vrpsx_errno_t (*stat)(                                              \
        TYPE *system,                                                   \
        char const *path,                                               \
        vrpsx_stat_t *buffer);                                          \
                                                                        \
    vrpsx_errno_t (*lstat)(                                             \
        TYPE *system,                                                   \
        char const *path,                                               \
        vrpsx_stat_t *buffer);                                          \
                                                                        \
    vrpsx_errno_t (*fstat)(                                             \
        TYPE *system,                                                   \
        vrpsx_fd_t fd,                                                  \
        vrpsx_stat_t *buffer);                                          \
                                                                        \
    vrpsx_errno_t (*mkdir)(                                             \
        TYPE *system,                                                   \
        char const *path,                                               \
        vrpsx_mode_t mode);                                             \
                                                                        \
    vrpsx_errno_t (*rmdir)(                                             \
        TYPE *system,                                                   \
        char const *path);                                              \
                                                                        \
    /* WARNING: NOT THREAD-SAFE.  Changing the current working          \
     * directory may undermine many other operations.                   \
     */                                                                 \
    vrpsx_errno_t (*chdir)(                                             \
        TYPE *system,                                                   \
        char const *path);                                              \
                                                                        \
    /* Allocates and constructs a new directory stream object.          \
     *                                                                  \
     * If successful, sets *dir to point to an interface                \
     * to the new stream; otherwise sets *dir to NULL.                  \
     * On success, the caller takes responsibility for eventually       \
     * invoking the "disown" operation on the directory stream.         \
     *                                                                  \
     * The old value of *dir is ignored.                                \
     */                                                                 \
    vrpsx_errno_t (*opendir)(                                           \
        TYPE *system,                                                   \
        char const *path,                                               \
        vrpsx_directory_t **dir);                                       \
                                                                        \
    /* The same deviations from POSIX as with "opendir";                \
     * see that operation.                                              \
     */                                                                 \
    vrpsx_errno_t (*fdopendir)(                                         \
        TYPE *system,                                                   \
        vrpsx_fd_t fd,                                                  \
        vrpsx_directory_t **dir);                                       \
                                                                        \
    /* There appears to be no POSIX standard for reading the table of   \
     * current filesystem mounts.  However, the "mount" and "umount"    \
     * programs of Solaris, FreeBSD, and Linux have enough commonality  \
     * to suggest that there is always a sequence of mounts, each of    \
     * which has the options described in the comments for              \
     * vrpsx_mount_t.                                                   \
     *                                                                  \
     * But automounters of various sorts complicate matters, in that:   \
     *                                                                  \
     *     1) they are not always configured using the "mount"          \
     *     command, and                                                 \
     *                                                                  \
     *     2) they create an indefinite collection of mounts with a     \
     *     single command.                                              \
     *                                                                  \
     * The general rule is that the returned iterator should describe   \
     * arguments that, if passed to the "mount" operation of this       \
     * interface in order, would recreate the current sequence of       \
     * mounts (assuming that the user had the necessary permissions to  \
     * actually establish the mounts, and that they were not already    \
     * established, etc.).  If possible, the implementation should      \
     * enhance the "mount" operation to handle not only ordinary        \
     * mounts, but also configuration of any automounters.  The         \
     * returned iteration should emit the direct mounts but omit        \
     * automatically mounted filesystems, instead emitting parameters   \
     * that configure the automounter that made those mounts.  These    \
     * should by encoded into the usual mount fields in some fashion,   \
     * for example by naming the automounter in the type field and      \
     * using the device and options to encode the automounter           \
     * configuration parameters.  For example, the device might         \
     * identify an LDAP server and the options the records to retrieve  \
     * from it to define the automatic mounts.  Where possible use the  \
     * spellings you would encounter while working with the most the    \
     * most relevant command-line program or text configuration file.   \
     * In any case, the lack of POSIX standardization means that you    \
     * should not be surprised by an enosys error, though of course an  \
     * application is not required to magically work around such lack   \
     * of support.                                                      \
     *                                                                  \
     * The preexisting value of *mounts is ignored.                     \
     * On failure sets *mounts to NULL.                                 \
     * Also sets *mounts to NULL upon successfully finding zero mounts. \
     *                                                                  \
     * WARNING: It is unsafe to change the mounts while iterating,      \
     * because in that case iteration might produce incorrect results.  \
     */                                                                 \
    vrpsx_errno_t (*get_mounts)(                                        \
        TYPE *system,                                                   \
        vrpsx_mount_t **mounts);                                        \
                                                                        \
    /* There appears to be no POSIX standard for mounting a             \
     * filesystem.  However, see the comments for "get_mounts":         \
     * as with that operation, we support the functionality common      \
     * to the "mount" programs of various UNIX-like operating systems.  \
     *                                                                  \
     * The option names should be the same ones you would use with the  \
     * "mount" program.                                                 \
     *                                                                  \
     * This function will always clear *options                         \
     * and take ownership of the option sequence,                       \
     * even in the case of an error.                                    \
     */                                                                 \
    vrpsx_errno_t (*mount)(                                             \
        TYPE *system,                                                   \
        char const *device,                                             \
        char const *root,                                               \
        char const *type,                                               \
        vrpsx_mount_option_t **options);                                \
                                                                        \
    /* There appears to be no POSIX standard for mounting a             \
     * filesystem.  However, see the comments for "get_mounts":         \
     * as with that operation, we support the functionality common      \
     * to the "umount" programs of various UNIX-like operating systems. \
     *                                                                  \
     * Unfortunately, "umount" typically only has single-character      \
     * switch options, instead of a general "unmount -o" mechanism.     \
     * We could list the switches themselves, but that is less than     \
     * readable, and therefore we recommend that the implementation     \
     * take the name of the corresponding bitmask macro, strip off      \
     * any namespace prefix, and lowercase the result.                  \
     *                                                                  \
     * This function will always clear *options                         \
     * and take ownership of the option sequence,                       \
     * even in the case of an error.                                    \
     */                                                                 \
    vrpsx_errno_t (*umount)(                                            \
        TYPE *system,                                                   \
        char const *device_or_root,                                     \
        vrpsx_mount_option_t **options);                                \
                                                                        \
    /* Marks the end of the function pointers for system operations.    \
     */                                                                 \
    vrpsx_system_operations_end_t const *system_operations_end;         \
                                                                        \
    /* constants from <stdlib.h>                                        \
     */                                                                 \
    int exit_failure;                                                   \
    int exit_success;                                                   \
                                                                        \
    /* Marks the end of the <stdlib.h> constants.                       \
     */                                                                 \
    vrpsx_system_stdlib_end_t const *system_stdlib_end;                 \
                                                                        \
    /* The error numbers defined by POSIX, together with:               \
     *                                                                  \
     *   - enomedium : removable storage device not present             \
     *                                                                  \
     * If one of our non-POSIX error number symbols is not meaningful   \
     * to the underlying system, then the implementation must assign    \
     * it a nonzero (possibly negative) value that differs from all     \
     * numbers that might arise from real errors, even if some of       \
     * those other numbers are not named by this interface.  Thus,      \
     * if a number describing an actual error is equal to a given       \
     * symbol, then that symbol meaningfully describes that error,      \
     * even if that symbol is not specified by POSIX.                   \
     *                                                                  \
     * Put another way, if the interface client looks only at zero      \
     * and the numbers of actual errors, then it will never observe     \
     * the fact that an extension symbol has no meaning on its system.  \
     * But if it calls error_acquire on the value of a non-meaningful   \
     * symbol, then the resulting text might be a generic fallback that \
     * does not describe the intent of that symbol.  (An implementation \
     * may choose to substitute its own text to mitigate that problem.) \
     *                                                                  \
     * Note that POSIX allows an implementation to assign the           \
     * same value to certain symbolic error number names, such          \
     * as EWOULDBLOCK and EAGAIN.  Like POSIX, we leave that            \
     * choice to the implementation, and therefore we cannot            \
     * use an enumeration here.                                         \
     *                                                                  \
     * Non-POSIX error number symbols may also share values,            \
     * though a symbol without a meaning on the underlying system       \
     * must not share its value with one that does have a meaning.      \
     *                                                                  \
     * Note that, like POSIX, we allow the implementation to            \
     * return other, implementation-defined error numbers.              \
     */                                                                 \
    vrpsx_errno_t e2big;                                                \
    vrpsx_errno_t eacces;                                               \
    vrpsx_errno_t eaddrinuse;                                           \
    vrpsx_errno_t eaddrnotavail;                                        \
    vrpsx_errno_t eafnosupport;                                         \
    vrpsx_errno_t eagain;                                               \
    vrpsx_errno_t ealready;                                             \
    vrpsx_errno_t ebadf;                                                \
    vrpsx_errno_t ebadmsg;                                              \
    vrpsx_errno_t ebusy;                                                \
    vrpsx_errno_t ecanceled;                                            \
    vrpsx_errno_t echild;                                               \
    vrpsx_errno_t econnaborted;                                         \
    vrpsx_errno_t econnrefused;                                         \
    vrpsx_errno_t econnreset;                                           \
    vrpsx_errno_t edeadlk;                                              \
    vrpsx_errno_t edestaddrreq;                                         \
    vrpsx_errno_t edom;                                                 \
    vrpsx_errno_t edquot;                                               \
    vrpsx_errno_t eexist;                                               \
    vrpsx_errno_t efault;                                               \
    vrpsx_errno_t efbig;                                                \
    vrpsx_errno_t ehostunreach;                                         \
    vrpsx_errno_t eidrm;                                                \
    vrpsx_errno_t eilseq;                                               \
    vrpsx_errno_t einprogress;                                          \
    vrpsx_errno_t eintr;                                                \
    vrpsx_errno_t einval;                                               \
    vrpsx_errno_t eio;                                                  \
    vrpsx_errno_t eisconn;                                              \
    vrpsx_errno_t eisdir;                                               \
    vrpsx_errno_t eloop;                                                \
    vrpsx_errno_t emfile;                                               \
    vrpsx_errno_t emlink;                                               \
    vrpsx_errno_t emsgsize;                                             \
    vrpsx_errno_t emultihop;                                            \
    vrpsx_errno_t enametoolong;                                         \
    vrpsx_errno_t enetdown;                                             \
    vrpsx_errno_t enetunreach;                                          \
    vrpsx_errno_t enfile;                                               \
    vrpsx_errno_t enobufs;                                              \
    vrpsx_errno_t enodata;                                              \
    vrpsx_errno_t enodev;                                               \
    vrpsx_errno_t enoent;                                               \
    vrpsx_errno_t enoexec;                                              \
    vrpsx_errno_t enolck;                                               \
    vrpsx_errno_t enolink;                                              \
    vrpsx_errno_t enomedium;                                            \
    vrpsx_errno_t enomem;                                               \
    vrpsx_errno_t enomsg;                                               \
    vrpsx_errno_t enoprotoopt;                                          \
    vrpsx_errno_t enospc;                                               \
    vrpsx_errno_t enosr;                                                \
    vrpsx_errno_t enostr;                                               \
    vrpsx_errno_t enosys;                                               \
    vrpsx_errno_t enotconn;                                             \
    vrpsx_errno_t enotdir;                                              \
    vrpsx_errno_t enotempty;                                            \
    vrpsx_errno_t enotsock;                                             \
    vrpsx_errno_t enotsup;                                              \
    vrpsx_errno_t enotty;                                               \
    vrpsx_errno_t enxio;                                                \
    vrpsx_errno_t eopnotsupp;                                           \
    vrpsx_errno_t eoverflow;                                            \
    vrpsx_errno_t eperm;                                                \
    vrpsx_errno_t epipe;                                                \
    vrpsx_errno_t eproto;                                               \
    vrpsx_errno_t eprotonosupport;                                      \
    vrpsx_errno_t eprototype;                                           \
    vrpsx_errno_t erange;                                               \
    vrpsx_errno_t erofs;                                                \
    vrpsx_errno_t espipe;                                               \
    vrpsx_errno_t esrch;                                                \
    vrpsx_errno_t estale;                                               \
    vrpsx_errno_t etime;                                                \
    vrpsx_errno_t etimedout;                                            \
    vrpsx_errno_t etxtbsy;                                              \
    vrpsx_errno_t ewouldblock;                                          \
    vrpsx_errno_t exdev;                                                \
                                                                        \
    /* Marks the end of the error numbers.                              \
     */                                                                 \
    vrpsx_system_errno_end_t const *system_errno_end;                   \
                                                                        \
    /* signal numbers                                                   \
     */                                                                 \
    vrpsx_signo_t sigabrt;                                              \
    vrpsx_signo_t sigalrm;                                              \
    vrpsx_signo_t sigbus;                                               \
    vrpsx_signo_t sigchld;                                              \
    vrpsx_signo_t sigcont;                                              \
    vrpsx_signo_t sigfpe;                                               \
    vrpsx_signo_t sighup;                                               \
    vrpsx_signo_t sigill;                                               \
    vrpsx_signo_t sigint;                                               \
    vrpsx_signo_t sigkill;                                              \
    vrpsx_signo_t sigpipe;                                              \
    vrpsx_signo_t sigpoll;                                              \
    vrpsx_signo_t sigprof;                                              \
    vrpsx_signo_t sigquit;                                              \
    vrpsx_signo_t sigsegv;                                              \
    vrpsx_signo_t sigstop;                                              \
    vrpsx_signo_t sigsys;                                               \
    vrpsx_signo_t sigterm;                                              \
    vrpsx_signo_t sigtrap;                                              \
    vrpsx_signo_t sigtstp;                                              \
    vrpsx_signo_t sigttin;                                              \
    vrpsx_signo_t sigttou;                                              \
    vrpsx_signo_t sigurg;                                               \
    vrpsx_signo_t sigusr1;                                              \
    vrpsx_signo_t sigusr2;                                              \
    vrpsx_signo_t sigvtalrm;                                            \
    vrpsx_signo_t sigxcpu;                                              \
    vrpsx_signo_t sigxfsz;                                              \
                                                                        \
    /* Marks the end of the signal numbers.                             \
     */                                                                 \
    vrpsx_system_signo_end_t const *system_signo_end;                   \
                                                                        \
    /* special signal-handling actions                                  \
     */                                                                 \
    vrpsx_sa_handler_t *sig_dfl;                                        \
    vrpsx_sa_handler_t *sig_err;                                        \
    vrpsx_sa_handler_t *sig_ign;                                        \
                                                                        \
    /* sizes of signal-related data structures                          \
     */                                                                 \
    size_t sizeof_sigset;                                               \
    size_t alignof_sigset;                                              \
    size_t sizeof_sigaction;                                            \
    size_t alignof_sigaction;                                           \
                                                                        \
    /* other signal-related constants                                   \
     */                                                                 \
    vrpsx_sa_flags_t sa_nocldstop;                                      \
    vrpsx_sa_flags_t sa_nocldwait;                                      \
    vrpsx_sa_flags_t sa_nodefer;                                        \
    vrpsx_sa_flags_t sa_onstack;                                        \
    vrpsx_sa_flags_t sa_resethand;                                      \
    vrpsx_sa_flags_t sa_restart;                                        \
    vrpsx_sa_flags_t sa_siginfo;                                        \
    int sig_block;                                                      \
    int sig_setmask;                                                    \
    int sig_unblock;                                                    \
                                                                        \
    /* Marks the end of the signal constants.                           \
     */                                                                 \
    vrpsx_system_signal_end_t const *system_signal_end;                 \
                                                                        \
    /* thread-related constants                                         \
     */                                                                 \
    int thread_create_detached;                                         \
    int thread_create_joinable;                                         \
    int thread_mutex_default;                                           \
    int thread_mutex_errorcheck;                                        \
    int thread_mutex_normal;                                            \
    int thread_mutex_recursive;                                         \
    int thread_process_private;                                         \
    int thread_process_shared;                                          \
    int thread_scope_process;                                           \
    int thread_scope_system;                                            \
                                                                        \
    /* Marks the end of the thread constants.                           \
     */                                                                 \
    vrpsx_system_thread_end_t const *system_thread_end;                 \
                                                                        \
    /* constants from <fcntl.h>                                         \
     */                                                                 \
    int f_dupfd;                                                        \
    int f_getfd;                                                        \
    int f_getfl;                                                        \
    int f_getlk;                                                        \
    int f_rdlck;                                                        \
    int f_setfd;                                                        \
    int f_setfl;                                                        \
    int f_setlk;                                                        \
    int f_setlkw;                                                       \
    int f_unlck;                                                        \
    int f_wrlck;                                                        \
    int fd_cloexec;                                                     \
    int o_accmode;                                                      \
    int o_append;                                                       \
    int o_creat;                                                        \
    int o_dsync;                                                        \
    int o_excl;                                                         \
    int o_noctty;                                                       \
    int o_nonblock;                                                     \
    int o_rdonly;                                                       \
    int o_rdwr;                                                         \
    int o_rsync;                                                        \
    int o_sync;                                                         \
    int o_trunc;                                                        \
    int o_wronly;                                                       \
                                                                        \
    /* Marks the end of the <fcntl.h> constants.                        \
     */                                                                 \
    vrpsx_system_fcntl_end_t const *system_fcntl_end;                   \
                                                                        \
    /* constants from <stat.h>                                          \
     */                                                                 \
    vrpsx_mode_t s_ifmt;                                                \
    vrpsx_mode_t s_ifblk;                                               \
    vrpsx_mode_t s_ifchr;                                               \
    vrpsx_mode_t s_ififo;                                               \
    vrpsx_mode_t s_ifreg;                                               \
    vrpsx_mode_t s_ifdir;                                               \
    vrpsx_mode_t s_iflnk;                                               \
    vrpsx_mode_t s_ifsock;                                              \
                                                                        \
    /* Marks the end of the <sys/stat.h> constants.                     \
     */                                                                 \
    vrpsx_system_stat_end_t const *system_stat_end;                     \
                                                                        \
    /* constants from <time.h>                                          \
     */                                                                 \
    vrpsx_clockid_t clock_monotonic;                                    \
    vrpsx_clockid_t clock_realtime;                                     \
                                                                        \
    /* Marks the end of the <time.h> constants.                         \
     */                                                                 \
    vrpsx_system_time_end_t const *system_time_end;                     \
                                                                        \
    /* constants from <limits.h>                                        \
     */                                                                 \
    size_t name_max;                                                    \
    size_t path_max;                                                    \
                                                                        \
    /* Marks the end of the <limits.h> constants.                       \
     */                                                                 \
    vrpsx_system_limits_end_t const *system_limits_end;                 \
                                                                        \
    /* arguments to "sysconf"                                           \
     */                                                                 \
    int _sc_2_c_bind;                                                   \
    int _sc_2_c_dev;                                                    \
    int _sc_2_char_term;                                                \
    int _sc_2_fort_dev;                                                 \
    int _sc_2_fort_run;                                                 \
    int _sc_2_localedef;                                                \
    int _sc_2_pbs;                                                      \
    int _sc_2_pbs_accounting;                                           \
    int _sc_2_pbs_checkpoint;                                           \
    int _sc_2_pbs_locate;                                               \
    int _sc_2_pbs_message;                                              \
    int _sc_2_pbs_track;                                                \
    int _sc_2_sw_dev;                                                   \
    int _sc_2_upe;                                                      \
    int _sc_2_version;                                                  \
    int _sc_advisory_info;                                              \
    int _sc_aio_listio_max;                                             \
    int _sc_aio_max;                                                    \
    int _sc_aio_prio_delta_max;                                         \
    int _sc_arg_max;                                                    \
    int _sc_asynchronous_io;                                            \
    int _sc_atexit_max;                                                 \
    int _sc_barriers;                                                   \
    int _sc_bc_base_max;                                                \
    int _sc_bc_dim_max;                                                 \
    int _sc_bc_scale_max;                                               \
    int _sc_bc_string_max;                                              \
    int _sc_child_max;                                                  \
    int _sc_clk_tck;                                                    \
    int _sc_clock_selection;                                            \
    int _sc_coll_weights_max;                                           \
    int _sc_cputime;                                                    \
    int _sc_delaytimer_max;                                             \
    int _sc_expr_nest_max;                                              \
    int _sc_fsync;                                                      \
    int _sc_getgr_r_size_max;                                           \
    int _sc_getpw_r_size_max;                                           \
    int _sc_host_name_max;                                              \
    int _sc_iov_max;                                                    \
    int _sc_ipv6;                                                       \
    int _sc_job_control;                                                \
    int _sc_line_max;                                                   \
    int _sc_login_name_max;                                             \
    int _sc_mapped_files;                                               \
    int _sc_memlock;                                                    \
    int _sc_memlock_range;                                              \
    int _sc_memory_protection;                                          \
    int _sc_message_passing;                                            \
    int _sc_monotonic_clock;                                            \
    int _sc_mq_open_max;                                                \
    int _sc_mq_prio_max;                                                \
    int _sc_ngroups_max;                                                \
    int _sc_open_max;                                                   \
    int _sc_page_size;                                                  \
    int _sc_pagesize;                                                   \
    int _sc_prioritized_io;                                             \
    int _sc_priority_scheduling;                                        \
    int _sc_raw_sockets;                                                \
    int _sc_re_dup_max;                                                 \
    int _sc_reader_writer_locks;                                        \
    int _sc_realtime_signals;                                           \
    int _sc_regexp;                                                     \
    int _sc_rtsig_max;                                                  \
    int _sc_saved_ids;                                                  \
    int _sc_sem_nsems_max;                                              \
    int _sc_sem_value_max;                                              \
    int _sc_semaphores;                                                 \
    int _sc_shared_memory_objects;                                      \
    int _sc_shell;                                                      \
    int _sc_sigqueue_max;                                               \
    int _sc_spawn;                                                      \
    int _sc_spin_locks;                                                 \
    int _sc_sporadic_server;                                            \
    int _sc_ss_repl_max;                                                \
    int _sc_stream_max;                                                 \
    int _sc_symloop_max;                                                \
    int _sc_synchronized_io;                                            \
    int _sc_thread_attr_stackaddr;                                      \
    int _sc_thread_attr_stacksize;                                      \
    int _sc_thread_cputime;                                             \
    int _sc_thread_destructor_iterations;                               \
    int _sc_thread_keys_max;                                            \
    int _sc_thread_prio_inherit;                                        \
    int _sc_thread_prio_protect;                                        \
    int _sc_thread_priority_scheduling;                                 \
    int _sc_thread_process_shared;                                      \
    int _sc_thread_robust_prio_inherit;                                 \
    int _sc_thread_robust_prio_protect;                                 \
    int _sc_thread_safe_functions;                                      \
    int _sc_thread_sporadic_server;                                     \
    int _sc_thread_stack_min;                                           \
    int _sc_thread_threads_max;                                         \
    int _sc_threads;                                                    \
    int _sc_timeouts;                                                   \
    int _sc_timer_max;                                                  \
    int _sc_timers;                                                     \
    int _sc_trace;                                                      \
    int _sc_trace_event_filter;                                         \
    int _sc_trace_event_name_max;                                       \
    int _sc_trace_inherit;                                              \
    int _sc_trace_log;                                                  \
    int _sc_trace_name_max;                                             \
    int _sc_trace_sys_max;                                              \
    int _sc_trace_user_event_max;                                       \
    int _sc_tty_name_max;                                               \
    int _sc_typed_memory_objects;                                       \
    int _sc_tzname_max;                                                 \
    int _sc_v6_ilp32_off32;                                             \
    int _sc_v6_ilp32_offbig;                                            \
    int _sc_v6_lp64_off64;                                              \
    int _sc_v6_lpbig_offbig;                                            \
    int _sc_v7_ilp32_off32;                                             \
    int _sc_v7_ilp32_offbig;                                            \
    int _sc_v7_lp64_off64;                                              \
    int _sc_v7_lpbig_offbig;                                            \
    int _sc_version;                                                    \
    int _sc_xopen_crypt;                                                \
    int _sc_xopen_enh_i18n;                                             \
    int _sc_xopen_realtime;                                             \
    int _sc_xopen_realtime_threads;                                     \
    int _sc_xopen_shm;                                                  \
    int _sc_xopen_streams;                                              \
    int _sc_xopen_unix;                                                 \
    int _sc_xopen_version;                                              \
                                                                        \
    /* Marks the end of the "sysconf" constants.                        \
     */                                                                 \
    vrpsx_system_sysconf_end_t const *system_sysconf_end;               \
                                                                        \
    /* arguments to "pathconf" and "fpathconf"                          \
     */                                                                 \
    int _pc_2_symlinks;                                                 \
    int _pc_alloc_size_min;                                             \
    int _pc_async_io;                                                   \
    int _pc_chown_restricted;                                           \
    int _pc_filesizebits;                                               \
    int _pc_link_max;                                                   \
    int _pc_max_canon;                                                  \
    int _pc_max_input;                                                  \
    int _pc_name_max;                                                   \
    int _pc_no_trunc;                                                   \
    int _pc_path_max;                                                   \
    int _pc_pipe_buf;                                                   \
    int _pc_prio_io;                                                    \
    int _pc_rec_incr_xfer_size;                                         \
    int _pc_rec_max_xfer_size;                                          \
    int _pc_rec_min_xfer_size;                                          \
    int _pc_rec_xfer_align;                                             \
    int _pc_symlink_max;                                                \
    int _pc_sync_io;                                                    \
    int _pc_timestamp_resolution;                                       \
    int _pc_vdisable;                                                   \
                                                                        \
    /* Marks the end of the "pathconf" and "fpathconf" constants.       \
     */                                                                 \
    vrpsx_system_pathconf_end_t const *system_pathconf_end;             \
                                                                        \
    /* End marker for documentation and to help detect initialization bugs. \
     */                                                                 \
    vrpsx_system_end_t const *system_end;

    VRPSX_SYSTEM_VTBL_MEMBERS(vrpsx_system_t)
} vrpsx_system_vtbl_t;


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* VRPSX_SYSTEM_H */
